@@ -36,7 +36,7 @@
 (defconstant compiled-debug-var-more-count-p           #b00000010)
 (defconstant compiled-debug-var-environment-live       #b00000100)
 (defconstant compiled-debug-var-save-loc-p             #b00001000)
-(defconstant compiled-debug-var-id-p                   #b00010000)
+(defconstant compiled-debug-var-same-name-p            #b00010000)
 (defconstant compiled-debug-var-minimal-p              #b00100000)
 (defconstant compiled-debug-var-deleted-p              #b01000000)
 (defconstant compiled-debug-var-indirect-p             #b10000000)
@@ -99,6 +99,7 @@
   (name (missing-arg) :type (or simple-string cons symbol) :read-only t)
   ;; a description of variable locations for this function, in alphabetical
   ;; order by name; or NIL if no information is available
+  ;; If only one variable is encoded then it's stored as is without a vector.
   ;;
   ;; The variable entries are alphabetically ordered. This ordering is
   ;; used in lifetime info to refer to variables: the first entry is
@@ -113,7 +114,8 @@
   ;;   * the variable ID, when it has one
   ;;   * SC-offset of primary location, if it has one
   ;;   * SC-offset of save location, if it has one
-  (vars nil :type (or simple-vector null))
+  ;; Can either be a single value or a vector for multiple values.
+  (vars nil)
   ;; a vector of the packed binary representation of the
   ;; COMPILED-DEBUG-BLOCKs in this function, in the order that the
   ;; blocks were emitted. The first block is the start of the
@@ -161,7 +163,9 @@
   ;; VARIABLES are all required arguments, and are in the order they
   ;; appear in the VARIABLES vector. In other words, :MINIMAL stands
   ;; in for a vector where every element holds its index.
-  (arguments nil :type (or (simple-array * (*)) (member :minimal nil)))
+  ;;
+  ;; Can either be a single value or a vector for multiple values.
+  (arguments nil)
   ;; There are three alternatives for this slot:
   ;;
   ;; a VECTOR
@@ -180,14 +184,59 @@
   (return-pc (missing-arg) :type sc-offset)
   #!-fp-and-pc-standard-save
   (old-fp (missing-arg) :type sc-offset)
+  ;; An integer which contains between 2 and 4 varint-encoded fields:
+  ;; START-PC -
   ;; The earliest PC in this function at which the environment is properly
   ;; initialized (arguments moved from passing locations, etc.)
-  (start-pc (missing-arg) :type index :read-only t)
+  ;; ELSEWHERE-PC -
   ;; The start of elsewhere code for this function (if any.)
-  (elsewhere-pc (missing-arg) :type index :read-only t)
-  (closure-save nil :type (or sc-offset null)  :read-only t)
-  #!+unwind-to-frame-and-call-vop
-  (bsp-save nil :type (or sc-offset null)  :read-only t))
+  ;; CLOSURE-SAVE, and BSP-SAVE.
+  (encoded-locs (missing-arg) :type unsigned-byte :read-only t))
+
+(defun cdf-encode-locs (start-pc elsewhere-pc closure-save
+                        #!+unwind-to-frame-and-call-vop bsp-save)
+  (dx-let ((storage (make-array (* 5 4) :element-type '(unsigned-byte 8))))
+    (let ((bytes (make-array (* 5 4) :fill-pointer 0 :displaced-to storage
+                             :element-type '(unsigned-byte 8))))
+      ;; ELSEWHERE is encoded first to simplify the C backtrace logic,
+      ;; which does not need access to any of the subsequent fields.
+      (write-var-integer elsewhere-pc bytes)
+      (write-var-integer start-pc bytes)
+      #!+unwind-to-frame-and-call-vop
+      (write-var-integer (if bsp-save (1+ bsp-save) 0) bytes)
+      ;; More often the BSP-SAVE is non-null than CLOSURE-SAVE is non-null,
+      ;; so the encoding is potentially smaller with CLOSURE-SAVE being last.
+      (when closure-save
+        (write-var-integer (1+ closure-save) bytes))
+      (integer-from-octets bytes))))
+
+(defun cdf-decode-locs (cdf)
+  (let ((encoding (compiled-debug-fun-encoded-locs cdf))
+        (input-pointer 0))
+    (flet ((decode-varint (&aux (accumulator 0) (shift 0))
+             (loop
+              (let ((byte (ldb (byte 8 input-pointer) encoding)))
+                (incf input-pointer 8)
+                (setf accumulator (logior accumulator (ash (logand byte #x7f) shift)))
+                (incf shift 7)
+                (unless (logtest byte #x80) (return accumulator))))))
+      (let ((elsewhere-pc (decode-varint))
+            (start-pc (decode-varint))
+            #!+unwind-to-frame-and-call-vop
+            ;; 0 -> NULL, 1 -> 0, ...
+            (bsp-save (let ((i (decode-varint))) (unless (zerop i) (1- i))))
+            (closure-save (let ((i (decode-varint))) (unless (zerop i) (1- i)))))
+        (values start-pc elsewhere-pc closure-save
+                #!+unwind-to-frame-and-call-vop bsp-save)))))
+
+(macrolet ((def (index name)
+             `(defun ,name (cdf)
+                (nth-value ,index (cdf-decode-locs cdf)))))
+  (def 0 compiled-debug-fun-start-pc)
+  (def 1 compiled-debug-fun-elsewhere-pc)
+  ;; Most compiled-debug-funs don't need these
+  (def 2 compiled-debug-fun-closure-save)
+  #!+unwind-to-frame-and-call-vop (def 3 compiled-debug-fun-bsp-save))
 
 (def!struct (compiled-debug-fun-optional (:include compiled-debug-fun)
                                          #-sb-xc-host (:pure t)
@@ -338,7 +387,8 @@
   ;; WHN 20000120
   (fun-map (missing-arg) :type simple-vector :read-only t)
   ;; Location contexts
-  (contexts nil :type (or simple-vector null) :read-only t)
+  ;; Either a simple-vector or a context if there's only one context.
+  (contexts nil :type t :read-only t)
   (tlf-number nil :type (or index null))
   (char-offset nil :type (or index null)))
 
