@@ -154,39 +154,47 @@
 
 ;;;; type-ish predicates
 
-;;; X may contain cycles -- a conservative approximation. This
-;;; occupies a somewhat uncomfortable niche between being fast for
-;;; common cases (we don't want to allocate a hash-table), and not
-;;; falling down to exponential behaviour for large trees (so we set
-;;; an arbitrady depth limit beyond which we punt).
-(defun maybe-cyclic-p (x &optional (depth-limit 12))
-  (and (listp x)
-       (labels ((safe-cddr (cons)
-                  (let ((cdr (cdr cons)))
-                    (when (consp cdr)
-                      (cdr cdr))))
-                (check-cycle (object seen depth)
-                  (when (and (consp object)
-                             (or (> depth depth-limit)
-                                 (member object seen)
-                                 (circularp object seen depth)))
-                    (return-from maybe-cyclic-p t)))
-                (circularp (list seen depth)
-                  ;; Almost regular circular list detection, with a twist:
-                  ;; we also check each element of the list for upward
-                  ;; references using CHECK-CYCLE.
-                  (do ((fast (cons (car list) (cdr list)) (safe-cddr fast))
-                       (slow list (cdr slow)))
-                      ((not (consp fast))
-                       ;; Not CDR-circular, need to check remaining CARs yet
-                       (do ((tail slow (and (cdr tail))))
-                           ((not (consp tail))
-                            nil)
-                         (check-cycle (car tail) (cons tail seen) (1+ depth))))
-                    (check-cycle (car slow) (cons slow seen) (1+ depth))
-                    (when (eq fast slow)
-                      (return t)))))
-         (circularp x (list x) 0))))
+;;; This is used for coalescing constants, check that the tree doesn't
+;;; have cycles and isn't too large.
+(defun coalesce-tree-p (x)
+  (let ((depth-limit 12)
+        (size-limit (expt 2 25)))
+    (declare (fixnum size-limit))
+    (and (consp x)
+         (labels ((safe-cddr (cons)
+                    (let ((cdr (cdr cons)))
+                      (when (consp cdr)
+                        (cdr cdr))))
+                  (check-cycle (cdr seen depth)
+                    (let ((object (car cdr)))
+                      (when (and (consp object)
+                                 (or (= depth depth-limit)
+                                     (memq object seen)
+                                     (let ((seen (cons cdr seen)))
+                                       (declare (dynamic-extent seen))
+                                       (recurse object seen
+                                                (truly-the fixnum (1+ depth))))))
+                        (return-from coalesce-tree-p))))
+                  (recurse (list seen depth)
+                    ;; Almost regular circular list detection, with a twist:
+                    ;; we also check each element of the list for upward
+                    ;; references using CHECK-CYCLE.
+                    (do ((fast (cdr list) (safe-cddr fast))
+                         (slow list (cdr slow)))
+                        ((not (consp fast))
+                         ;; Not CDR-circular, need to check remaining CARs yet
+                         (do ((tail slow (cdr tail)))
+                             ((not (consp tail)))
+                           (check-cycle tail seen depth)))
+                      (check-cycle slow seen depth)
+                      (when (or (eq fast slow)
+                                (zerop (setf size-limit
+                                             (truly-the fixnum
+                                                        (1- size-limit)))))
+                        (return-from coalesce-tree-p)))))
+           (declare (inline check-cycle))
+           (recurse x (list x) 0)
+           t))))
 
 ;;; Is X a (possibly-improper) list of at least N elements?
 (declaim (ftype (function (t index)) list-of-length-at-least-p))
@@ -873,7 +881,11 @@ NOTE: This interface is experimental and subject to change."
 
 ;;;; various operations on names
 
-;;; Is NAME a legal function name?
+;;; Is NAME a legal variable/function name?
+(declaim (inline legal-variable-name-p))
+(defun legal-variable-name-p (name)
+  (typep name '(and symbol (not keyword))))
+
 (declaim (inline legal-fun-name-p))
 (defun legal-fun-name-p (name)
   (values (valid-function-name-p name)))
@@ -887,7 +899,7 @@ NOTE: This interface is experimental and subject to change."
     (error 'simple-type-error
            :datum name
            :expected-type 'function-name
-           :format-control "invalid function name: ~S"
+           :format-control "Invalid function name: ~S"
            :format-arguments (list name))))
 
 ;;; Given a function name, return the symbol embedded in it.
@@ -1267,6 +1279,9 @@ NOTE: This interface is experimental and subject to change."
                     (deprecation-info-version info))
               (deprecation-info-replacements info)))))
 
+;;; Without a proclaimed type, the call is "untrusted" and so the compiler
+;;; would generate a post-call check that the function did not return.
+(declaim (ftype (function (t t t t t) nil) deprecation-error))
 (defun deprecation-error (software version namespace name replacements)
   #-sb-xc-host(declare (optimize allow-non-returning-tail-call))
   (error 'deprecation-error
