@@ -157,7 +157,7 @@
   x)
 
 (!define-fop 66 :not-host (fop-misc-trap)
-  (%primitive sb!c:make-unbound-marker))
+  (make-unbound-marker))
 
 (!define-fop 76 (fop-character ((:operands char-code)))
   (code-char char-code))
@@ -204,7 +204,7 @@
       (dotimes (i n-slots)
         (let ((val (svref stack (+ ptr i)))
               (slot-name (pop slot-names)))
-          (if (eq val sb!pcl:+slot-unbound+)
+          (if (unbound-marker-p val)
               ;; SLOT-MAKUNBOUND-USING-CLASS might do something nonstandard.
               (slot-makunbound obj slot-name)
               (setf (slot-value obj slot-name) val))))))
@@ -440,7 +440,7 @@
         (declare (fixnum size))
         (let ((ptr (fop-stack-pop-n stack size)))
           (replace res stack :start2 ptr))
-        res)))
+        (logically-readonlyize res))))
 
 ;; No MAKE-ARRAY-HEADER on host
 (!define-fop 89 :not-host (fop-array (vec))
@@ -479,7 +479,8 @@
          (vector (if (and (= widetag sb!vm:simple-vector-widetag)
                           (= words 0))
                      #()
-                     (allocate-vector widetag length words))))
+                     (logically-readonlyize
+                      (allocate-vector widetag length words)))))
     (declare (type index length bytes words)
              (type word bits))
     (read-n-bytes (fasl-input-stream) vector 0 bytes)
