@@ -42,6 +42,9 @@
     ((double-float)
      (,function ,var))))
 
+(defun handle-complex (form)
+  `((((foreach (complex double-float) (complex single-float) (complex rational)))
+     ,form)))
 ) ; EVAL-WHEN
 
 #!+x86 ;; for constant folding
@@ -98,30 +101,42 @@
 ;;;; power functions
 
 (defun exp (number)
-  #!+sb-doc
   "Return e raised to the power NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %exp number)
-    ((complex)
+    (handle-complex
      (* (exp (realpart number))
         (cis (imagpart number))))))
 
 ;;; INTEXP -- Handle the rational base, integer power case.
-
-(declaim (type (or integer null) *intexp-maximum-exponent*))
-(defparameter *intexp-maximum-exponent* nil)
-
 ;;; This function precisely calculates base raised to an integral
 ;;; power. It separates the cases by the sign of power, for efficiency
 ;;; reasons, as powers can be calculated more efficiently if power is
 ;;; a positive integer. Values of power are calculated as positive
 ;;; integers, and inverted if negative.
 (defun intexp (base power)
-  (when (and *intexp-maximum-exponent*
-             (> (abs power) *intexp-maximum-exponent*))
-    (error "The absolute value of ~S exceeds ~S."
-            power '*intexp-maximum-exponent*))
-  (cond ((minusp power)
+  (cond ((eql base 1)
+         base)
+        ((eql base -1)
+         (if (evenp power)
+             1
+             base))
+        ((ratiop base)
+         (let ((den (denominator base))
+               (num (numerator base)))
+           (if (minusp power)
+               (let ((negated (- power)))
+                 (cond ((eql num 1)
+                        (intexp den negated))
+                       ((eql num -1)
+                        (intexp (- den) negated))
+                       (t
+                        (build-ratio (intexp den negated)
+                                     (intexp num negated)))))
+               (build-ratio (intexp num power)
+                            (intexp den power)))))
+        ((minusp power)
          (/ (intexp base (- power))))
         ((eql base 2)
          (ash 1 power))
@@ -140,8 +155,8 @@
 ;;; result. We also separate the complex-real and real-complex cases
 ;;; from the general complex case.
 (defun expt (base power)
-  #!+sb-doc
   "Return BASE raised to the POWER."
+  (declare (explicit-check))
   (if (zerop power)
     (if (and (zerop base) (floatp power))
         (error 'arguments-out-of-domain-error
@@ -291,9 +306,12 @@
          (real-expt base power 'double-float))
         ;; Handle (expt <complex> <rational>), except the case dealt with
         ;; in the first clause above, (expt <(complex rational)> <integer>).
-        (((foreach (complex rational) (complex single-float)
-                   (complex double-float))
-          rational)
+        (((foreach (complex rational))
+          ratio)
+         (* (expt (abs base) power)
+            (cis (* power (phase base)))))
+        (((foreach (complex single-float) (complex double-float))
+          (foreach fixnum (or bignum ratio)))
          (* (expt (abs base) power)
             (cis (* power (phase base)))))
         ;; The next three clauses handle (expt <real> <complex>).
@@ -339,8 +357,8 @@
                     2.0d0))))))
 
 (defun log (number &optional (base nil base-p))
-  #!+sb-doc
   "Return the logarithm of NUMBER in the base BASE, which defaults to e."
+  (declare (explicit-check))
   (if base-p
       (cond
         ((zerop base)
@@ -387,30 +405,33 @@
          (complex-log number)))))
 
 (defun sqrt (number)
-  #!+sb-doc
   "Return the square root of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (((foreach fixnum bignum ratio))
      (if (minusp number)
-         (complex-sqrt number)
+         (complex 0f0
+                  (coerce (%sqrt (- (coerce number 'double-float))) 'single-float))
          (coerce (%sqrt (coerce number 'double-float)) 'single-float)))
     (((foreach single-float double-float))
      (if (minusp number)
-         (complex-sqrt (complex number))
+         (complex (coerce 0.0 '(dispatch-type number))
+                  (coerce (%sqrt (- (coerce number 'double-float)))
+                          '(dispatch-type number)))
          (coerce (%sqrt (coerce number 'double-float))
                  '(dispatch-type number))))
-     ((complex)
-      (complex-sqrt number))))
+    ((complex)
+     (complex-sqrt number))))
 
 ;;;; trigonometic and related functions
 
 (defun abs (number)
-  #!+sb-doc
   "Return the absolute value of the number."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (((foreach single-float double-float fixnum rational))
      (abs number))
-    ((complex)
+    (handle-complex
      (let ((rx (realpart number))
            (ix (imagpart number)))
        (etypecase rx
@@ -424,66 +445,71 @@
           (%hypot rx (truly-the double-float ix))))))))
 
 (defun phase (number)
-  #!+sb-doc
   "Return the angle part of the polar representation of a complex number.
   For complex numbers, this is (atan (imagpart number) (realpart number)).
   For non-complex positive numbers, this is 0. For non-complex negative
   numbers this is PI."
-  (etypecase number
-    (rational
+  (declare (explicit-check))
+  (number-dispatch ((number number))
+    ((rational)
      (if (minusp number)
          (coerce pi 'single-float)
          0.0f0))
-    (single-float
+    ((single-float)
      (if (minusp (float-sign number))
          (coerce pi 'single-float)
          0.0f0))
-    (double-float
+    ((double-float)
      (if (minusp (float-sign number))
          (coerce pi 'double-float)
          0.0d0))
-    (complex
+    (handle-complex
      (atan (imagpart number) (realpart number)))))
 
 (defun sin (number)
-  #!+sb-doc
   "Return the sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %sin number)
-    ((complex)
+    (handle-complex
      (let ((x (realpart number))
            (y (imagpart number)))
        (complex (* (sin x) (cosh y))
                 (* (cos x) (sinh y)))))))
 
 (defun cos (number)
-  #!+sb-doc
   "Return the cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %cos number)
-    ((complex)
+    (handle-complex
      (let ((x (realpart number))
            (y (imagpart number)))
        (complex (* (cos x) (cosh y))
                 (- (* (sin x) (sinh y))))))))
 
 (defun tan (number)
-  #!+sb-doc
   "Return the tangent of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %tan number)
-    ((complex)
-     (complex-tan number))))
+    (handle-complex
+     ;; tan z = -i * tanh(i*z)
+     (let* ((result (complex-tanh (complex (- (imagpart number))
+                                           (realpart number)))))
+       (complex (imagpart result)
+                (- (realpart result)))))))
 
 (defun cis (theta)
-  #!+sb-doc
   "Return cos(Theta) + i sin(Theta), i.e. exp(i Theta)."
-  (declare (type real theta))
-  (complex (cos theta) (sin theta)))
+  (declare (explicit-check ))
+  (number-dispatch ((theta real))
+    (((foreach single-float double-float rational))
+     (complex (cos theta) (sin theta)))))
 
 (defun asin (number)
-  #!+sb-doc
   "Return the arc sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      (if (or (> number 1) (< number -1))
@@ -499,8 +525,8 @@
      (complex-asin number))))
 
 (defun acos (number)
-  #!+sb-doc
   "Return the arc cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      (if (or (> number 1) (< number -1))
@@ -516,8 +542,8 @@
      (complex-acos number))))
 
 (defun atan (y &optional (x nil xp))
-  #!+sb-doc
   "Return the arc tangent of Y if X is omitted or Y/X if X is supplied."
+  (declare (explicit-check))
   (if xp
       (flet ((atan2 (y x)
                (declare (type double-float y x)
@@ -552,46 +578,46 @@
 ;;; complex numbers can also lose big.
 
 (defun sinh (number)
-  #!+sb-doc
   "Return the hyperbolic sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %sinh number)
-    ((complex)
+    (handle-complex
      (let ((x (realpart number))
            (y (imagpart number)))
        (complex (* (sinh x) (cos y))
                 (* (cosh x) (sin y)))))))
 
 (defun cosh (number)
-  #!+sb-doc
   "Return the hyperbolic cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %cosh number)
-    ((complex)
+    (handle-complex
      (let ((x (realpart number))
            (y (imagpart number)))
        (complex (* (cosh x) (cos y))
                 (* (sinh x) (sin y)))))))
 
 (defun tanh (number)
-  #!+sb-doc
   "Return the hyperbolic tangent of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %tanh number)
     ((complex)
      (complex-tanh number))))
 
 (defun asinh (number)
-  #!+sb-doc
   "Return the hyperbolic arc sine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     (handle-reals %asinh number)
     ((complex)
      (complex-asinh number))))
 
 (defun acosh (number)
-  #!+sb-doc
   "Return the hyperbolic arc cosine of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      ;; acosh is complex if number < 1
@@ -607,8 +633,8 @@
      (complex-acosh number))))
 
 (defun atanh (number)
-  #!+sb-doc
   "Return the hyperbolic arc tangent of NUMBER."
+  (declare (explicit-check))
   (number-dispatch ((number number))
     ((rational)
      ;; atanh is complex if |number| > 1
@@ -661,13 +687,12 @@
 ;;;;   complex-asin
 ;;;;   complex-asinh
 ;;;;   complex-atan
-;;;;   complex-tan
 ;;;;
 ;;;; utility functions:
-;;;;   scalb logb
+;;;;   logb
 ;;;;
 ;;;; internal functions:
-;;;;    square coerce-to-complex-type cssqs complex-log-scaled
+;;;;    square coerce-to-complex-type cssqs
 ;;;;
 ;;;; references:
 ;;;;   Kahan, W. "Branch Cuts for Complex Elementary Functions, or Much
@@ -708,20 +733,12 @@
   (declare (double-float x))
   (* x x))
 
-;;; original CMU CL comment, apparently re. SCALB and LOGB and
+;;; original CMU CL comment, apparently re. LOGB and
 ;;; perhaps CSSQS:
 ;;;   If you have these functions in libm, perhaps they should be used
 ;;;   instead of these Lisp versions. These versions are probably good
 ;;;   enough, especially since they are portable.
 
-;;; Compute 2^N * X without computing 2^N first. (Use properties of
-;;; the underlying floating-point format.)
-(declaim (inline scalb))
-(defun scalb (x n)
-  (declare (type double-float x)
-           (type double-float-exponent n))
-  (scale-float x n))
-
 ;;; This is like LOGB, but X is not infinity and non-zero and not a
 ;;; NaN, so we can always return an integer.
 (declaim (inline logb-finite))
@@ -779,6 +796,7 @@
 #!+long-float (eval-when (:compile-toplevel :load-toplevel :execute)
                 (error "needs work for long float support"))
 (defun cssqs (z)
+  (declare (muffle-conditions t))
   (let ((x (float (realpart z) 1d0))
         (y (float (imagpart z) 1d0)))
     ;; Would this be better handled using an exception handler to
@@ -815,8 +833,8 @@
               ;; least one is non-zero.. Thus logb returns a nice
               ;; integer.
               (let ((k (- (logb-finite (max (abs x) (abs y))))))
-                (values (+ (square (scalb x k))
-                           (square (scalb y k)))
+                (values (+ (square (scale-float x k))
+                           (square (scale-float y k)))
                         (- k))))
              (t
               (values rho 0)))))))
@@ -840,14 +858,11 @@
           (y (float (imagpart z) 1.0d0))
           (eta 0d0)
           (nu 0d0))
-      (declare (double-float x y eta nu))
-
-      (locally
-         ;; space 0 to get maybe-inline functions inlined.
-         (declare (optimize (speed 3) (space 0)))
-
+      (declare (double-float x y eta nu)
+               ;; get maybe-inline functions inlined.
+               (optimize (space 0)))
       (if (not (float-nan-p x))
-          (setf rho (+ (scalb (abs x) (- k)) (sqrt rho))))
+          (setf rho (+ (scale-float (abs x) (- k)) (sqrt rho))))
 
       (cond ((oddp k)
              (setf k (ash k -1)))
@@ -855,25 +870,25 @@
              (setf k (1- (ash k -1)))
              (setf rho (+ rho rho))))
 
-      (setf rho (scalb (sqrt rho) k))
+      (setf rho (scale-float (sqrt rho) k))
 
       (setf eta rho)
       (setf nu y)
 
       (when (/= rho 0d0)
-            (when (not (float-infinity-p (abs nu)))
-                  (setf nu (/ (/ nu rho) 2d0)))
-            (when (< x 0d0)
-                  (setf eta (abs nu))
-                  (setf nu (float-sign y rho))))
-       (coerce-to-complex-type eta nu z)))))
+        (when (not (float-infinity-p (abs nu)))
+          (setf nu (/ (/ nu rho) 2d0)))
+        (when (< x 0d0)
+          (setf eta (abs nu))
+          (setf nu (float-sign y rho))))
+      (coerce-to-complex-type eta nu z))))
 
-;;; Compute log(2^j*z).
+;;; log of Z = log |Z| + i * arg Z
 ;;;
-;;; This is for use with J /= 0 only when |z| is huge.
-(defun complex-log-scaled (z j)
-  (declare (type (or rational complex) z)
-           (fixnum j))
+;;; Z may be any number, but the result is always a complex.
+(defun complex-log (z)
+  (declare (muffle-conditions t))
+  (declare (type (or rational complex) z))
   ;; The constants t0, t1, t2 should be evaluated to machine
   ;; precision.  In addition, Kahan says the accuracy of log1p
   ;; influences the choices of these constants but doesn't say how to
@@ -903,31 +918,25 @@
       (let ((beta (max (abs x) (abs y)))
             (theta (min (abs x) (abs y))))
         (coerce-to-complex-type (if (and (zerop k)
-                 (< t0 beta)
-                 (or (<= beta t1)
-                     (< rho t2)))
-                                  (/ (%log1p (+ (* (- beta 1.0d0)
-                                       (+ beta 1.0d0))
-                                    (* theta theta)))
-                                     2d0)
-                                  (+ (/ (log rho) 2d0)
-                                     (* (+ k j) ln2)))
+                                         (< t0 beta)
+                                         (or (<= beta t1)
+                                             (< rho t2)))
+                                    (/ (%log1p (+ (* (- beta 1.0d0)
+                                                     (+ beta 1.0d0))
+                                                  (* theta theta)))
+                                       2d0)
+                                    (+ (/ (log rho) 2d0)
+                                       (* k ln2)))
                                 (atan y x)
                                 z)))))
 
-;;; log of Z = log |Z| + i * arg Z
-;;;
-;;; Z may be any number, but the result is always a complex.
-(defun complex-log (z)
-  (declare (type (or rational complex) z))
-  (complex-log-scaled z 0))
-
 ;;; KLUDGE: Let us note the following "strange" behavior. atanh 1.0d0
 ;;; is +infinity, but the following code returns approx 176 + i*pi/4.
 ;;; The reason for the imaginary part is caused by the fact that arg
 ;;; i*y is never 0 since we have positive and negative zeroes. -- rtoy
 ;;; Compute atanh z = (log(1+z) - log(1-z))/2.
 (defun complex-atanh (z)
+  (declare (muffle-conditions t))
   (declare (type (or rational complex) z))
   (let* (;; constants
          (theta (/ (sqrt most-positive-double-float) 4.0d0))
@@ -985,6 +994,7 @@
 
 ;;; Compute tanh z = sinh z / cosh z.
 (defun complex-tanh (z)
+  (declare (muffle-conditions t))
   (declare (type (or rational complex) z))
   (let ((x (float (realpart z) 1.0d0))
         (y (float (imagpart z) 1.0d0)))
@@ -1095,14 +1105,3 @@
          (result (complex-atanh iz)))
     (complex (imagpart result)
              (- (realpart result)))))
-
-;;; Compute tan z = -i * tanh(i * z)
-;;;
-;;; Z may be any number, but the result is always a complex.
-(defun complex-tan (z)
-  (declare (type (or rational complex) z))
-  ;; tan z = -i * tanh(i*z)
-  (let* ((iz (complex (- (imagpart z)) (realpart z)))
-         (result (complex-tanh iz)))
-    (complex (imagpart result)
-             (- (realpart result)))))
