@@ -216,6 +216,7 @@
 (defun args-exhausted (control-string offset)
   (format-error-at control-string offset "No more arguments"))
 
+(defvar *format-gensym-counter*)
 (defun expand-control-string (string)
   (let* ((string (etypecase string
                    (simple-string
@@ -223,6 +224,7 @@
                    (string
                     (coerce string 'simple-string))))
          (*default-format-error-control-string* string)
+         (*format-gensym-counter* 0)
          (directives (tokenize-control-string string)))
     `(block nil
        ,@(expand-directive-list directives))))
@@ -294,7 +296,13 @@
       `(,*expander-next-arg-macro*
         ,*default-format-error-control-string*
         ,(or offset *default-format-error-offset*))
-      (let ((symbol (sb!xc:gensym "FORMAT-ARG")))
+      (let ((symbol
+             (without-package-locks
+                 (package-symbolicate
+                  (load-time-value (find-package "SB!FORMAT") t)
+                  "FORMAT-ARG"
+                  (write-to-string (incf *format-gensym-counter*)
+                                   :pretty nil :base 10 :radix nil)))))
         (push (cons symbol (or offset *default-format-error-offset*))
               *simple-args*)
         symbol)))
@@ -957,7 +965,7 @@
               (nthcdr (1+ posn) directives)))))
 
 (def-complex-format-directive #\} ()
-  (format-error "no corresponding open brace"))
+  (format-error "No corresponding open brace"))
 
 ;;;; format directives and support functions for justification
 
@@ -1011,7 +1019,7 @@
      remaining)))
 
 (def-complex-format-directive #\> ()
-  (format-error "no corresponding open bracket"))
+  (format-error "No corresponding open bracket"))
 
 (defun parse-format-logical-block
        (segments colonp first-semi close params string end)
@@ -1218,13 +1226,14 @@
                                 (subseq foo (1+ slash) (1- end)))))
          (first-colon (position #\: name))
          (second-colon (if first-colon (position #\: name :start (1+ first-colon))))
-         (package-name (if first-colon
-                           (subseq name 0 first-colon)
-                           "COMMON-LISP-USER"))
-         (package (or (find-package package-name)
+         (package
+            (if (not first-colon)
+                (load-time-value (find-package "COMMON-LISP-USER") t)
+                (let ((package-name (subseq name 0 first-colon)))
+                  (or (find-package package-name)
                       ;; FIXME: should be PACKAGE-ERROR? Could we just
                       ;; use FIND-UNDELETED-PACKAGE-OR-LOSE?
-                      (format-error "No package named ~S" package-name))))
+                      (format-error "No package named ~S" package-name))))))
     (intern (cond
               ((and second-colon (= second-colon (1+ first-colon)))
                (subseq name (1+ second-colon)))
