@@ -61,7 +61,7 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defun %with-standard-io-syntax (function)
   (declare (type function function))
-  (let ((*package* (find-package "COMMON-LISP-USER"))
+  (let ((*package* (load-time-value (find-package "COMMON-LISP-USER") t))
         (*print-array* t)
         (*print-base* 10)
         (*print-case* :upcase)
@@ -195,7 +195,7 @@ variable: an unreadable object representing the error is printed instead.")
                     (stream (%make-finite-base-string-output-stream string)))
                (declare (inline %make-finite-base-string-output-stream))
                (declare (truly-dynamic-extent stream))
-               (output-integer object stream)
+               (output-integer object stream *print-base* *print-radix*)
                (%shrink-vector string
                                (finite-base-string-output-stream-pointer stream)))))))
     ;; Could do something for other numeric types, symbols, ...
@@ -442,9 +442,9 @@ variable: an unreadable object representing the error is printed instead.")
               (let ((prefix (or (car (rassoc package (package-%local-nicknames current)))
                                 (package-name package))))
                 (output-token prefix))
-              (if (nth-value 1 (find-external-symbol name package))
-                  (write-char #\: stream)
-                  (write-string "::" stream))))))
+              (write-char #\: stream)
+              (when (eql (find-external-symbol name package) 0)
+                (write-char #\: stream))))))
         (output-token name)))))
 
 ;;;; escaping symbols
@@ -823,62 +823,37 @@ variable: an unreadable object representing the error is printed instead.")
         (incf length)))
     (write-char #\) stream)))
 
-(defun output-unreadable-vector-readably (vector stream)
-  (declare (vector vector))
-  (write-string "#." stream)
-  (write `(coerce ,(coerce vector '(vector t))
-                  '(simple-array ,(array-element-type vector) (*)))
-         :stream stream))
-
 (defmethod print-object ((vector vector) stream)
- (let ((readably *print-readably*))
-  (cond ((stringp vector)
-         (let ((coerce-p
-                (and readably (not (typep vector '(vector character))))))
-           (cond ((and coerce-p (not *read-eval*))
-                  (print-not-readable-error vector stream))
+  (let ((readably *print-readably*))
+    (cond ((stringp vector)
+           (cond ((and readably (not (typep vector '(vector character))))
+                  (output-unreadable-array-readably vector stream))
                  ((or *print-escape* readably)
-                  (when coerce-p
-                    ;; OUTPUT-UNREADABLE-VECTOR-READABLY would output each char
-                    ;; in #\c syntax. In addition to wasting time coercing to a
-                    ;; general vector, it's not nice looking.
-                    (write-string "#.(" stream)
-                    (write 'coerce :stream stream) ; package-qualify / casify as needed
-                    (write-char #\Space stream))
                   (write-char #\" stream)
                   (quote-string vector stream)
-                  (write-char #\" stream)
-                  (when coerce-p
-                    (write-char #\Space stream)
-                    (write (cond #!+sb-unicode
-                                 ((base-string-p vector)
-                                  ''base-string)
-                                 (t
-                                  `'(vector ,(array-element-type vector)))) :stream stream)
-                    (write-char #\) stream)))
+                  (write-char #\" stream))
                  (t
-                  (write-string vector stream)))))
-        ((not (or *print-array* readably))
-         (output-terse-array vector stream))
-        ((bit-vector-p vector)
-         (write-string "#*" stream)
-         (dovector (bit vector)
-           ;; (Don't use OUTPUT-OBJECT here, since this code
-           ;; has to work for all possible *PRINT-BASE* values.)
-           (write-char (if (zerop bit) #\0 #\1) stream)))
-        ((or (not readably) (array-readably-printable-p vector))
-         (descend-into (stream)
-                       (write-string "#(" stream)
-                       (dotimes (i (length vector))
-                         (unless (zerop i)
-                           (write-char #\space stream))
-                         (punt-print-if-too-long i stream)
-                         (output-object (aref vector i) stream))
-                       (write-string ")" stream)))
-        (*read-eval*
-         (output-unreadable-vector-readably vector stream))
-        (t
-         (print-not-readable-error vector stream)))))
+                  (write-string vector stream))))
+          ((not (or *print-array* readably))
+           (output-terse-array vector stream))
+          ((bit-vector-p vector)
+           (write-string "#*" stream)
+           (dovector (bit vector)
+             ;; (Don't use OUTPUT-OBJECT here, since this code
+             ;; has to work for all possible *PRINT-BASE* values.)
+             (write-char (if (zerop bit) #\0 #\1) stream)))
+          ((or (not readably) (array-readably-printable-p vector))
+           (descend-into (stream)
+             (write-string "#(" stream)
+             (dotimes (i (length vector))
+               (unless (zerop i)
+                 (write-char #\space stream))
+               (punt-print-if-too-long i stream)
+               (output-object (aref vector i) stream))
+             (write-string ")" stream)))
+
+          (t
+           (output-unreadable-array-readably vector stream)))))
 
 ;;; This function outputs a string quoting characters sufficiently
 ;;; so that someone can read it in again. Basically, put a slash in
@@ -921,42 +896,55 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; Convert an array into a list that can be used with MAKE-ARRAY's
 ;;; :INITIAL-CONTENTS keyword argument.
 (defun listify-array (array)
-  (with-array-data ((data array) (start) (end))
-    (declare (ignore end))
-    (labels ((listify (dimensions index)
-               (if (null dimensions)
-                   (aref data index)
-                   (let* ((dimension (car dimensions))
-                          (dimensions (cdr dimensions))
-                          (count (reduce #'* dimensions)))
-                     (loop for i below dimension
-                           collect (listify dimensions index)
-                           do (incf index count))))))
-      (listify (array-dimensions array) start))))
+  (flet ((compact (seq)
+           (typecase array
+             (string
+              (coerce seq '(simple-array character (*))))
+             ((array bit)
+              (coerce seq 'bit-vector))
+             (t
+              seq))))
+    (if (typep array '(or string bit-vector))
+        (compact array)
+        (with-array-data ((data array) (start) (end))
+          (declare (ignore end))
+          (labels ((listify (dimensions index)
+                     (if (null dimensions)
+                         (aref data index)
+                         (let* ((dimension (car dimensions))
+                                (dimensions (cdr dimensions))
+                                (count (reduce #'* dimensions)))
+                           (loop for i below dimension
+                                 for list = (listify dimensions index)
+                                 collect (if (and dimensions
+                                                  (null (cdr dimensions)))
+                                             (compact list)
+                                             list)
+                                 do (incf index count))))))
+            (listify (array-dimensions array) start))))))
 
+;;; Use nonstandard #A(dimensions element-type contents)
+;;; to avoid using #.
 (defun output-unreadable-array-readably (array stream)
-  (write-string "#." stream)
-  (write `(make-array ',(array-dimensions array)
-                      :element-type ',(array-element-type array)
-                      :initial-contents ',(listify-array array))
-         :stream stream))
+  (let ((array (list* (array-dimensions array)
+                      (array-element-type array)
+                      (listify-array array))))
+    (write-string "#A" stream)
+    (write array :stream stream)
+    nil))
 
 ;;; Output the readable #A form of an array.
 (defun output-array-guts (array stream)
   (cond ((or (not *print-readably*)
              (array-readably-printable-p array))
          (write-char #\# stream)
-         (let ((*print-base* 10)
-               (*print-radix* nil))
-           (output-integer (array-rank array) stream))
+         (output-integer (array-rank array) stream 10 nil)
          (write-char #\A stream)
          (with-array-data ((data array) (start) (end))
            (declare (ignore end))
            (sub-output-array-guts data (array-dimensions array) stream start)))
-        (*read-eval*
-         (output-unreadable-array-readably array stream))
         (t
-         (print-not-readable-error array stream))))
+         (output-unreadable-array-readably array stream))))
 
 (defun sub-output-array-guts (array dimensions stream index)
   (declare (type (simple-array * (*)) array) (fixnum index))
@@ -1098,15 +1086,15 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;; This gets both a method and a specifically named function
 ;;; since the latter is called from a few places.
-(defmethod print-object ((object integer) stream) (output-integer object stream))
-(defun output-integer (integer stream)
-  (let ((base *print-base*))
-    (cond (*print-radix*
-           (unless (= base 10) (%output-radix base stream))
-           (%output-integer-in-base integer base stream)
-           (when (= base 10) (write-char #\. stream)))
-          (t
-           (%output-integer-in-base integer base stream)))))
+(defmethod print-object ((object integer) stream)
+  (output-integer object stream *print-base* *print-radix*))
+(defun output-integer (integer stream base radixp)
+  (cond (radixp
+         (unless (= base 10) (%output-radix base stream))
+         (%output-integer-in-base integer base stream)
+         (when (= base 10) (write-char #\. stream)))
+        (t
+         (%output-integer-in-base integer base stream))))
 
 (defmethod print-object ((ratio ratio) stream)
   (let ((base *print-base*))
@@ -1709,38 +1697,40 @@ variable: an unreadable object representing the error is printed instead.")
 ;;;; catch-all for unknown things
 
 (defmethod print-object ((object t) stream)
- (flet ((output-it (stream)
-  (print-unreadable-object (object stream :identity t)
-    (let ((lowtag (lowtag-of object)))
-      (case lowtag
-        (#.sb!vm:other-pointer-lowtag
-          (let ((widetag (widetag-of object)))
-            (case widetag
-              (#.sb!vm:value-cell-widetag
-               (write-string "value cell " stream)
-               (output-object (value-cell-ref object) stream))
-              (t
-               (write-string "unknown pointer object, widetag=" stream)
-               (let ((*print-base* 16) (*print-radix* t))
-                 (output-integer widetag stream))))))
-        ((#.sb!vm:fun-pointer-lowtag
-          #.sb!vm:instance-pointer-lowtag
-          #.sb!vm:list-pointer-lowtag)
-         (write-string "unknown pointer object, lowtag=" stream)
-         (let ((*print-base* 16) (*print-radix* t))
-           (output-integer lowtag stream)))
-        (t
-         (case (widetag-of object)
-           (#.sb!vm:unbound-marker-widetag
-            (write-string "unbound marker" stream))
-           (t
-            (write-string "unknown immediate object, lowtag=" stream)
-            (let ((*print-base* 2) (*print-radix* t))
-              (output-integer lowtag stream))
-            (write-string ", widetag=" stream)
-            (let ((*print-base* 16) (*print-radix* t))
-              (output-integer (widetag-of object) stream))))))))))
-  (if *print-pretty*
+  (flet ((output-it (stream)
+          (when (eq object sb!pcl:+slot-unbound+)
+            (print-unreadable-object (object stream)
+              ;; If specifically the unbound marker with 0 data,
+              ;; as opposed to any other unbound marker.
+              (write-string "unbound" stream))
+            (return-from output-it))
+           (print-unreadable-object (object stream :identity t)
+             (let ((lowtag (lowtag-of object)))
+               (case lowtag
+                 (#.sb!vm:other-pointer-lowtag
+                  (let ((widetag (widetag-of object)))
+                    (case widetag
+                      (#.sb!vm:value-cell-widetag
+                       (write-string "value cell " stream)
+                       (output-object (value-cell-ref object) stream))
+                      (t
+                       (write-string "unknown pointer object, widetag=" stream)
+                       (output-integer widetag stream 16 t)))))
+                 ((#.sb!vm:fun-pointer-lowtag
+                   #.sb!vm:instance-pointer-lowtag
+                   #.sb!vm:list-pointer-lowtag)
+                  (write-string "unknown pointer object, lowtag=" stream)
+                  (output-integer lowtag stream 16 t))
+                (t
+                 (case (widetag-of object)
+                   (#.sb!vm:unbound-marker-widetag
+                    (write-string "unbound marker" stream))
+                   (t
+                    (write-string "unknown immediate object, lowtag=" stream)
+                    (output-integer lowtag stream 2 t)
+                    (write-string ", widetag=" stream)
+                    (output-integer (widetag-of object) stream 16 t)))))))))
+    (if *print-pretty*
       ;; This block might not be necessary. Not sure, probably can't hurt.
-      (pprint-logical-block (stream nil) (output-it stream))
-      (output-it stream))))
+        (pprint-logical-block (stream nil) (output-it stream))
+        (output-it stream))))
