@@ -14,10 +14,22 @@
 
 ;;;; type format database
 
-(defun room-info-type-name (info)
-  (if (specialized-array-element-type-properties-p info)
-      (saetp-primitive-type-name info)
-      (room-info-name info)))
+(eval-when (:compile-toplevel :load-toplevel :execute)
+
+  (def!struct (room-info (:constructor make-room-info (name kind)))
+    ;; the name of this type
+    (name nil :type symbol :read-only t)
+    ;; kind of type (how to reconstitute an object)
+    (kind (missing-arg)
+          :type (member :other :tiny-other :closure :instance :list
+                        :code :vector-nil :weak-pointer)
+          :read-only t))
+  (!set-load-form-method room-info (:xc))
+
+  (defun room-info-type-name (info)
+    (if (specialized-array-element-type-properties-p info)
+        (saetp-primitive-type-name info)
+        (room-info-name info))))
 
 (eval-when (:compile-toplevel :execute)
 
@@ -31,26 +43,22 @@
                (not (member widetag '(t nil)))
                (not (eq name 'weak-pointer)))
       (setf (svref *meta-room-info* (symbol-value widetag))
-            (make-room-info :name name
-                            :kind (if (member name '(fdefn symbol))
-                                      :tiny-other
-                                      :other))))))
+            (make-room-info name (if (member name '(fdefn symbol))
+                                     :tiny-other
+                                     :other))))))
 
 (dolist (code (list #!+sb-unicode complex-character-string-widetag
                     complex-base-string-widetag simple-array-widetag
                     complex-bit-vector-widetag complex-vector-widetag
                     complex-array-widetag complex-vector-nil-widetag))
   (setf (svref *meta-room-info* code)
-        (make-room-info :name 'array-header
-                        :kind :other)))
+        (make-room-info 'array-header :other)))
 
 (setf (svref *meta-room-info* bignum-widetag)
-      (make-room-info :name 'bignum
-                      :kind :other))
+      (make-room-info 'bignum :other))
 
 (setf (svref *meta-room-info* closure-widetag)
-      (make-room-info :name 'closure
-                      :kind :closure))
+      (make-room-info 'closure :closure))
 
 (dotimes (i (length *specialized-array-element-type-properties*))
   (let ((saetp (aref *specialized-array-element-type-properties* i)))
@@ -58,27 +66,21 @@
       (setf (svref *meta-room-info* (saetp-typecode saetp)) saetp))))
 
 (setf (svref *meta-room-info* simple-array-nil-widetag)
-      (make-room-info :name 'simple-array-nil
-                      :kind :vector-nil))
+      (make-room-info 'simple-array-nil :vector-nil))
 
 (setf (svref *meta-room-info* code-header-widetag)
-      (make-room-info :name 'code
-                      :kind :code))
+      (make-room-info 'code :code))
 
 (setf (svref *meta-room-info* instance-widetag)
-      (make-room-info :name 'instance
-                      :kind :instance))
+      (make-room-info 'instance :instance))
 
 (setf (svref *meta-room-info* funcallable-instance-widetag)
-      (make-room-info :name 'funcallable-instance
-                      :kind :closure))
+      (make-room-info 'funcallable-instance :closure))
 
 (setf (svref *meta-room-info* weak-pointer-widetag)
-      (make-room-info :name 'weak-pointer
-                      :kind :weak-pointer))
+      (make-room-info 'weak-pointer :weak-pointer))
 
-(let ((cons-info (make-room-info :name 'cons
-                                 :kind :list)))
+(let ((cons-info (make-room-info 'cons :list)))
   ;; A cons consists of two words, both of which may be either a
   ;; pointer or immediate data.  According to the runtime this means
   ;; either a fixnum, a character, an unbound-marker, a single-float
@@ -110,7 +112,7 @@
 
 ) ; EVAL-WHEN
 
-(defparameter *room-info*
+(define-load-time-global *room-info*
   ;; SAETP instances don't dump properly from XC (or possibly
   ;; normally), and we'd rather share structure with the master copy
   ;; if we can anyway, so...
@@ -670,7 +672,7 @@
            (pages-so-far 0)
            (count-so-far 0)
            (last-page 0))
-      (declare (type (unsigned-byte 32) last-page start)
+      (declare (type word last-page start)
                (fixnum pages-so-far count-so-far pagesize))
       (labels ((note-conses (x)
                  (unless (or (atom x) (gethash x printed-conses))
@@ -687,10 +689,10 @@
                  (return-from print-allocated-objects (values)))
 
                (unless count
-                 (let ((this-page (* (the (values (unsigned-byte 32) t)
+                 (let ((this-page (* (the (values word t)
                                        (truncate addr pagesize))
                                      pagesize)))
-                   (declare (type (unsigned-byte 32) this-page))
+                   (declare (type word this-page))
                    (when (/= this-page last-page)
                      (when (< pages-so-far pages)
                        ;; FIXME: What is this? (ERROR "Argh..")? or
@@ -804,43 +806,79 @@
   (declare (type (or (eql :all) spaces) space))
   (unless *ignore-after*
     (setq *ignore-after* (cons 1 2)))
-  (flet ((maybe-call (fun obj)
-           (when (valid-obj space obj)
-             (funcall fun obj))))
-    (map-allocated-objects
-     (lambda (obj obj-type size)
-       (declare (ignore obj-type size))
-       (typecase obj
-         (cons
-          (when (or (eq (car obj) object)
-                    (eq (cdr obj) object))
-            (maybe-call fun obj)))
-         (instance
-          (when (or (eq (%instance-layout obj) object)
-                    (do-instance-tagged-slot (i obj)
-                      (when (eq (%instance-ref obj i) object)
-                        (return t))))
-            (maybe-call fun obj)))
-         (code-component
-          (let ((length (code-header-words obj)))
-            (do ((i code-constants-offset (1+ i)))
-                ((= i length))
-              (when (eq (code-header-ref obj i) object)
-                (maybe-call fun obj)
-                (return)))))
-         (simple-vector
-          (dotimes (i (length obj))
-            (when (eq (svref obj i) object)
-              (maybe-call fun obj)
-              (return))))
-         (symbol
-          (when (or (eq (symbol-name obj) object)
-                    (eq (symbol-package obj) object)
-                    (eq (symbol-info obj) object)
-                    (and (boundp obj)
-                         (eq (symbol-value obj) object)))
-            (maybe-call fun obj)))))
-     space)))
+  (flet ((ref-p (this widetag nwords) ; return T if 'this' references object
+           (when (listp this)
+             (return-from ref-p
+               (or (eq (car this) object) (eq (cdr this) object))))
+           (case widetag
+             ;; purely boxed objects
+             ((#.ratio-widetag #.complex-widetag #.value-cell-widetag
+               #.symbol-widetag #.weak-pointer-widetag
+               #.simple-array-widetag #.simple-vector-widetag
+               #.complex-array-widetag #.complex-vector-widetag
+               #.complex-bit-vector-widetag #.complex-vector-nil-widetag
+               #.complex-base-string-widetag
+               #!+sb-unicode #.complex-character-string-widetag))
+             ;; mixed boxed/unboxed objects
+             (#.code-header-widetag
+              (dotimes (i (code-n-entries this))
+                (let ((f (%code-entry-point this i)))
+                  (when (or (eq f object)
+                            (eq (%simple-fun-name f) object)
+                            (eq (%simple-fun-arglist f) object)
+                            (eq (%simple-fun-type f) object)
+                            (eq (%simple-fun-info f) object))
+                    (return-from ref-p t))))
+              (setq nwords (code-header-words this)))
+             (#.instance-widetag
+              (return-from ref-p
+                (or (eq (%instance-layout this) object)
+                    (do-instance-tagged-slot (i this)
+                      (when (eq (%instance-ref this i) object)
+                        (return t))))))
+             (#.funcallable-instance-widetag
+              (let ((l (%funcallable-instance-layout this)))
+                (when (eq l object)
+                  (return-from ref-p t))
+                (let ((bitmap (layout-bitmap l)))
+                  (unless (eql bitmap -1)
+                    ;; tagged slots precede untagged slots,
+                    ;; so integer-length is the count of tagged slots.
+                    (setq nwords (1+ (integer-length bitmap)))))))
+             (#.closure-widetag
+              (when (eq (%closure-fun this) object)
+                (return-from ref-p t)))
+             (#.fdefn-widetag
+              #!+immobile-code
+              (when (eq (make-lisp-obj
+                         (alien-funcall
+                          (extern-alien "fdefn_raw_referent" (function unsigned unsigned))
+                          (logandc2 (get-lisp-obj-address this) lowtag-mask)))
+                        object)
+                (return-from ref-p t))
+              ;; Without immobile-code the 'raw-addr' slot either holds the same thing
+              ;; as the 'fun' slot, or holds a trampoline address. We'll overlook the
+              ;; minor issue that due to concurrent writes, two representations of the
+              ;; allegedly same referent may diverge; thus the last slot is skipped
+              ;; even if it refers to a different simple-fun.
+              (decf nwords))
+             (t
+              (return-from ref-p nil)))
+           ;; gencgc has WITHOUT-GCING in map-allocated-objects over dynamic space,
+           ;; so we don't have to pin each object inside REF-P.
+           (#!+cheneygc with-pinned-objects #!+cheneygc (this)
+            #!-cheneygc progn
+            (do ((sap (int-sap (logandc2 (get-lisp-obj-address this) lowtag-mask)))
+                 (i (* (1- nwords) n-word-bytes) (- i n-word-bytes)))
+                ((<= i 0) nil)
+              (when (eq (sap-ref-lispobj sap i) object)
+                (return t))))))
+    (let ((fun (%coerce-callable-to-fun fun)))
+      (dx-flet ((mapfun (obj widetag size)
+                  (when (and (ref-p obj widetag (/ size n-word-bytes))
+                             (valid-obj space obj))
+                    (funcall fun obj))))
+        (map-allocated-objects #'mapfun space)))))
 
 (defun list-referencing-objects (space object)
   (collect ((res))
