@@ -49,7 +49,6 @@
 (progn
   (define-alien-routine wrapped-environ (* c-string))
   (defun posix-environ ()
-    #+sb-doc
     "Return the Unix environment (\"man environ\") as a list of SIMPLE-STRINGs."
     (c-strings->string-list (wrapped-environ))))
 
@@ -114,7 +113,6 @@
 
 #-win32
 (defun waitpid (pid &optional do-not-hang check-for-stopped)
-  #+sb-doc
   "Return any available status information on child process with PID."
   (multiple-value-bind (pid status)
       (c-waitpid pid
@@ -151,11 +149,10 @@
                      (not (zerop (ldb (byte 1 7) status)))))))))
 
 ;;;; process control stuff
-(defvar *active-processes* nil
-  #+sb-doc
+(define-load-time-global *active-processes* nil
   "List of process structures for all active processes.")
 
-(defvar *active-processes-lock*
+(define-load-time-global *active-processes-lock*
   (sb-thread:make-mutex :name "Lock for active processes."))
 
 ;;; *ACTIVE-PROCESSES* can be accessed from multiple threads so a
@@ -170,13 +167,15 @@
   %status             ; either :RUNNING, :STOPPED, :EXITED, or :SIGNALED
   %exit-code          ; either exit code or signal
   core-dumped         ; T if a core image was dumped
-  #-win32 pty                 ; stream to child's pty, or NIL
+  #-win32 pty         ; stream to child's pty, or NIL
   input               ; stream to child's input, or NIL
   output              ; stream from child's output, or NIL
   error               ; stream from child's error output, or NIL
   status-hook         ; closure to call when PROC changes status
   plist               ; a place for clients to stash things
-  cookie)             ; list of the number of pipes from the subproc
+  cookie              ; list of the number of pipes from the subproc
+  #+win32
+  copiers)            ; list of sb-win32::io-copier
 
 (defmethod print-object ((process process) stream)
   (print-unreadable-object (process stream :type t)
@@ -186,79 +185,50 @@
          (format stream "~S ~S" (process-pid process) status)))
     process))
 
-#+sb-doc
-(setf (documentation 'process-p 'function)
-      "T if OBJECT is a PROCESS, NIL otherwise.")
-
-#+sb-doc
-(setf (documentation 'process-pid 'function) "The pid of the child process.")
-
 #+win32
 (define-alien-routine ("GetExitCodeProcess" get-exit-code-process)
     int
   (handle unsigned) (exit-code unsigned :out))
 
 (defun process-exit-code (process)
-  #+sb-doc
   "Return the exit code of PROCESS."
   (or (process-%exit-code process)
       (progn (get-processes-status-changes)
              (process-%exit-code process))))
 
 (defun process-status (process)
-  #+sb-doc
   "Return the current status of PROCESS.  The result is one of :RUNNING,
    :STOPPED, :EXITED, or :SIGNALED."
   (get-processes-status-changes)
   (process-%status process))
 
-#+sb-doc
 (setf (documentation 'process-exit-code 'function)
-      "The exit code or the signal of a stopped process.")
-
-#+sb-doc
-(setf (documentation 'process-core-dumped 'function)
-      "T if a core image was dumped by the process.")
-
-#+sb-doc
-(setf (documentation 'process-pty 'function)
-      "The pty stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-input 'function)
-      "The input stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-output 'function)
-      "The output stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-error 'function)
-      "The error stream of the process or NIL.")
-
-#+sb-doc
-(setf (documentation 'process-status-hook  'function)
-      "A function that is called when PROCESS changes its status.
-The function is called with PROCESS as its only argument.")
-
-#+sb-doc
-(setf (documentation 'process-plist  'function)
-      "A place for clients to stash things.")
+      "The exit code or the signal of a stopped process."
+      (documentation 'process-core-dumped 'function)
+      "T if a core image was dumped by the process."
+      (documentation 'process-pty 'function)
+      "The pty stream of the process or NIL."
+      (documentation 'process-input 'function)
+      "The input stream of the process or NIL."
+      (documentation 'process-output 'function)
+      "The output stream of the process or NIL."
+      (documentation 'process-error 'function)
+      "The error stream of the process or NIL."
+      (documentation 'process-status-hook 'function) "A function that is called when PROCESS changes its status.
+The function is called with PROCESS as its only argument."
+      (documentation 'process-plist 'function)
+      "A place for clients to stash things."
+      (documentation 'process-p 'function)
+      "T if OBJECT is a PROCESS, NIL otherwise."
+      (documentation 'process-pid 'function) "The pid of the child process.")
 
 (defun process-wait (process &optional check-for-stopped)
-  #+sb-doc
   "Wait for PROCESS to quit running for some reason. When
 CHECK-FOR-STOPPED is T, also returns when PROCESS is stopped. Returns
 PROCESS."
   (declare (ignorable check-for-stopped))
   #+win32
-  (let ((pid (process-pid process)))
-    (when (and pid (plusp pid))
-      (without-interrupts
-        (do ()
-            ((= 0
-                (with-local-interrupts
-                  (sb-win32:wait-object-or-signal pid))))))))
+  (sb-win32::win32-process-wait process)
   #-win32
   (loop
       (case (process-status process)
@@ -288,7 +258,6 @@ PROCESS."
 
 #-win32
 (defun process-kill (process signal &optional (whom :pid))
-  #+sb-doc
   "Hand SIGNAL to PROCESS. If WHOM is :PID, use the kill Unix system call. If
    WHOM is :PROCESS-GROUP, use the killpg Unix system call. If WHOM is
    :PTY-PROCESS-GROUP deliver the signal to whichever process group is
@@ -298,15 +267,11 @@ PROCESS."
                 (process-pid process))
                (:pty-process-group
                 (find-current-foreground-process process)))))
-    (multiple-value-bind
-          (okay errno)
-        (case whom
-          ((:process-group)
-           (sb-unix:unix-killpg pid signal))
-          (t
-           (sb-unix:unix-kill pid signal)))
-      (cond ((not okay)
-             (values nil errno))
+    (let ((result (if (eq whom :process-group)
+                      (sb-unix:unix-killpg pid signal)
+                      (sb-unix:unix-kill pid signal))))
+      (cond ((not (eql result 0))
+             (values nil (sb-unix::get-errno)))
             ((and (eql pid (process-pid process))
                   (= signal sb-unix:sigcont))
              (setf (process-%status process) :running)
@@ -318,7 +283,6 @@ PROCESS."
              t)))))
 
 (defun process-alive-p (process)
-  #+sb-doc
   "Return T if PROCESS is still alive, NIL otherwise."
   (let ((status (process-status process)))
     (if (or (eq status :running)
@@ -327,7 +291,6 @@ PROCESS."
         nil)))
 
 (defun process-close (process)
-  #+sb-doc
   "Close all streams connected to PROCESS and stop maintaining the
 status slot."
   (macrolet ((frob (stream abort)
@@ -374,7 +337,8 @@ status slot."
                            (when pid
                              (multiple-value-bind (ok code)
                                  (sb-win32::get-exit-code-process pid)
-                               (when (and (plusp ok) (/= code 259))
+                               (when (and (plusp ok)
+                                          (/= code sb-win32::still-active))
                                  (setf (process-%status proc) :exited
                                        (process-%exit-code proc) code)
                                  (when (process-status-hook proc)
@@ -398,9 +362,7 @@ status slot."
 ;;; list of file descriptors to close when RUN-PROGRAM returns in the parent
 (defvar *close-in-parent* nil)
 
-;;; list of handlers installed by RUN-PROGRAM.  FIXME: nothing seems
-;;; to set this.
-#-win32
+;;; list of handlers installed by RUN-PROGRAM.
 (defvar *handlers-installed* nil)
 
 ;;; Find an unused pty. Return three values: the file descriptor for
@@ -610,26 +572,26 @@ status slot."
               do (write-char #\\ stream)))
   (write-char #\" stream))
 
+#+win32
+(defun prepare-args (args escape)
+  (with-simple-output-to-string (str)
+    (loop for (arg . rest) on args
+          do
+          (cond ((and escape
+                      (find-if (lambda (c) (find c '(#\Space #\Tab #\")))
+                               arg))
+                 (escape-arg arg str))
+                (t
+                 (write-string arg str)))
+          (when rest
+            (write-char #\Space str)))))
+
+#-win32
 (defun prepare-args (args)
-  (cond #-win32
-        ((every #'simple-string-p args)
-         args)
-        #-win32
-        (t
-         (loop for arg in args
-               collect (coerce arg 'simple-string)))
-        #+win32
-        (t
-         (with-simple-output-to-string (str)
-           (loop for (arg . rest) on args
-                 do
-                 (cond ((find-if (lambda (c) (find c '(#\Space #\Tab #\")))
-                                 arg)
-                        (escape-arg arg str))
-                       (t
-                        (princ arg str)))
-                 (when rest
-                   (write-char #\Space str)))))))
+  (if (every #'simple-string-p args)
+      args
+      (loop for arg in args
+            collect (coerce arg 'simple-string))))
 
 ;;; FIXME: There shouldn't be two semiredundant versions of the
 ;;; documentation. Since this is a public extension function, the
@@ -692,10 +654,10 @@ status slot."
                     (if-error-exists :error)
                     status-hook
                     (external-format :default)
-                    directory)
-  #+sb-doc
+                    directory
+                    #+win32 (escape-arguments t))
   #.(concatenate
-     'string
+     'base-string
      ;; The Texinfoizer is sensitive to whitespace, so mind the
      ;; placement of the #-win32 pseudosplicings.
      "RUN-PROGRAM creates a new process specified by the PROGRAM
@@ -778,20 +740,22 @@ Users Manual for details about the PROCESS structure.
       The external-format to use for :INPUT, :OUTPUT, and :ERROR :STREAMs.
    :DIRECTORY
       Specifies the directory in which the program should be run.
-      NIL (the default) means the directory is unchanged.")
+      NIL (the default) means the directory is unchanged.
+
+   Windows specific options:
+   :ESCAPE-ARGUMENTS (default T)
+      Controls escaping of the arguments passed to CreateProcess.")
   (when (and env-p environment-p)
     (error "can't specify :ENV and :ENVIRONMENT simultaneously"))
   (let* (;; Clear various specials used by GET-DESCRIPTOR-FOR to
          ;; communicate cleanup info.
          *close-on-error*
          *close-in-parent*
-         ;; Some other binding used only on non-Win32.  FIXME:
-         ;; nothing seems to set this.
-         #-win32 *handlers-installed*
+         *handlers-installed*
          ;; Establish PROC at this level so that we can return it.
          proc
          (progname (native-namestring program))
-         (args (prepare-args (cons progname args)))
+         (args (prepare-args (cons progname args) #+win32 escape-arguments))
          (directory (and directory (native-namestring directory)))
          ;; Gag.
          (cookie (list 0)))
@@ -805,13 +769,13 @@ Users Manual for details about the PROCESS structure.
                       `(multiple-value-bind (,fd ,stream)
                            ,(ecase which
                               ((:input :output)
-                               `(get-descriptor-for ,@args))
+                               `(get-descriptor-for ,which ,@args))
                               (:error
                                `(if (eq ,(first args) :output)
                                     ;; kludge: we expand into
                                     ;; hard-coded symbols here.
                                     (values stdout output-stream)
-                                    (get-descriptor-for ,@args))))
+                                    (get-descriptor-for ,which ,@args))))
                          (unless ,fd
                            (return-from run-program))
                          ,@body))
@@ -872,15 +836,9 @@ Users Manual for details about the PROCESS structure.
                                   :status-hook status-hook
                                   :cookie cookie
                                   #-win32 :pty #-win32 pty-stream
-                                  :%status #-win32 :running
-                                           #+win32 (if wait
-                                                       :exited
-                                                       :running)
-                                  :pid #-win32 child
-                                       #+win32 (if wait
-                                                   nil
-                                                   child)
-                                  #+win32 :%exit-code #+win32 (and wait child)))
+                                  :%status :running
+                                  :pid child
+                                  #+win32 :copiers #+win32 *handlers-installed*))
                            (push proc *active-processes*))))
                      ;; Report the error outside the lock.
                      (case child
@@ -901,10 +859,10 @@ Users Manual for details about the PROCESS structure.
         #-win32
         (dolist (handler *handlers-installed*)
           (remove-fd-handler handler)))
-      #-win32
       (when (and wait proc)
         (unwind-protect
              (process-wait proc)
+          #-win32
           (dolist (handler *handlers-installed*)
             (remove-fd-handler handler)))))
     proc))
@@ -912,6 +870,7 @@ Users Manual for details about the PROCESS structure.
 ;;; Install a handler for any input that shows up on the file
 ;;; descriptor. The handler reads the data and writes it to the
 ;;; stream.
+#-win32
 (defun copy-descriptor-to-stream (descriptor stream cookie external-format)
   (incf (car cookie))
   (let* ((handler nil)
@@ -1007,9 +966,16 @@ Users Manual for details about the PROCESS structure.
                     (t
                      (incf read-end count)
                      (funcall copy-fun))))))))
-    #-win32
     (push handler *handlers-installed*)))
 
+#+win32
+(defun copy-descriptor-to-stream (descriptor stream cookie external-format)
+  (declare (ignore cookie))
+  (push (sb-win32::make-io-copier :pipe descriptor
+                                  :stream stream
+                                  :external-format external-format)
+        *handlers-installed*))
+
 ;;; FIXME: something very like this is done in SB-POSIX to treat
 ;;; streams as file descriptor designators; maybe we can combine these
 ;;; two?  Additionally, as we have a couple of user-defined streams
@@ -1042,8 +1008,7 @@ Users Manual for details about the PROCESS structure.
 ;;; Find a file descriptor to use for object given the direction.
 ;;; Returns the descriptor. If object is :STREAM, returns the created
 ;;; stream as the second value.
-(defun get-descriptor-for (object
-                           cookie
+(defun get-descriptor-for (argument object cookie
                            &rest keys
                            &key direction (external-format :default) wait
                            &allow-other-keys)
@@ -1054,20 +1019,24 @@ Users Manual for details about the PROCESS structure.
   ;; user-defined stream classes, we can end up trying to copy
   ;; arbitrarily much data into the temp file, and so are liable to
   ;; run afoul of disk quotas or to choke on small /tmp file systems.
-  (flet ((make-temp-fd ()
-           (multiple-value-bind (fd name/errno)
-               (sb-unix:sb-mkstemp (format nil "~a/.run-program-XXXXXX"
-                                           (get-temporary-directory))
-                                   #o0600)
-             (unless fd
-               (error "could not open a temporary file: ~A"
-                      (strerror name/errno)))
-             ;; Can't unlink an open file on Windows
-             #-win32
-             (unless (sb-unix:unix-unlink name/errno)
-               (sb-unix:unix-close fd)
-               (error "failed to unlink ~A" name/errno))
-             fd)))
+  (labels ((fail (format &rest arguments)
+             (error "~s error processing ~s argument:~% ~?" 'run-program argument format arguments))
+           (make-temp-fd ()
+             (multiple-value-bind (fd name/errno)
+                 (sb-unix:sb-mkstemp (format nil "~a/.run-program-XXXXXX"
+                                             (get-temporary-directory))
+                                     #o0600)
+               (unless fd
+                 (fail "could not open a temporary file: ~A"
+                       (strerror name/errno)))
+               #+win32
+               (setf (sb-win32::inheritable-handle-p fd) t)
+               ;; Can't unlink an open file on Windows
+               #-win32
+               (unless (sb-unix:unix-unlink name/errno)
+                 (sb-unix:unix-close fd)
+                 (fail "failed to unlink ~A" name/errno))
+               fd)))
     (let ((dev-null #.(coerce #-win32 "/dev/null" #+win32 "nul" 'base-string)))
       (cond ((eq object t)
              ;; No new descriptor is needed.
@@ -1083,10 +1052,11 @@ Users Manual for details about the PROCESS structure.
                                       (:input sb-unix:o_rdonly)
                                       (:output sb-unix:o_wronly)
                                       (t sb-unix:o_rdwr))
-                                    #o666)
+                                    #o666
+                                    #+win32 :overlapped #+win32 nil)
                (unless fd
-                 (error "~@<couldn't open ~S: ~2I~_~A~:>"
-                        dev-null (strerror errno)))
+                 (fail "~@<couldn't open ~S: ~2I~_~A~:>"
+                       dev-null (strerror errno)))
                #+win32
                (setf (sb-win32::inheritable-handle-p fd) t)
                (push fd *close-in-parent*)
@@ -1094,7 +1064,7 @@ Users Manual for details about the PROCESS structure.
             ((eq object :stream)
              (multiple-value-bind (read-fd write-fd) (sb-unix:unix-pipe)
                (unless read-fd
-                 (error "couldn't create pipe: ~A" (strerror write-fd)))
+                 (fail "couldn't create pipe: ~A" (strerror write-fd)))
                #+win32
                (setf (sb-win32::inheritable-handle-p read-fd)
                      (eq direction :input)
@@ -1120,24 +1090,28 @@ Users Manual for details about the PROCESS structure.
                  (t
                     (sb-unix:unix-close read-fd)
                     (sb-unix:unix-close write-fd)
-                    (error "Direction must be either :INPUT or :OUTPUT, not ~S."
+                    (fail "Direction must be either :INPUT or :OUTPUT, not ~S."
                            direction)))))
             ((or (pathnamep object) (stringp object))
              ;; GET-DESCRIPTOR-FOR uses &allow-other-keys, so rather
              ;; than munge the &rest list for OPEN, just disable keyword
              ;; validation there.
-             (with-open-stream (file (apply #'open object :allow-other-keys t
-                                            keys))
-               (when file
-                 (multiple-value-bind
-                       (fd errno)
-                     (sb-unix:unix-dup (fd-stream-fd file))
-                   (cond (fd
-                          (push fd *close-in-parent*)
-                          (values fd nil))
-                         (t
-                          (error "couldn't duplicate file descriptor: ~A"
-                                 (strerror errno))))))))
+             (with-open-stream (file (or (apply #'open object
+                                                :allow-other-keys t
+                                                #+win32 :overlapped #+win32 nil
+                                                keys)
+                                         ;; :if-input-does-not-exist nil
+                                         ;; can result in this
+                                         (return-from get-descriptor-for)))
+               (multiple-value-bind
+                     (fd errno)
+                   (sb-unix:unix-dup (fd-stream-fd file))
+                 (cond (fd
+                        (push fd *close-in-parent*)
+                        (values fd nil))
+                       (t
+                        (fail "couldn't duplicate file descriptor: ~A"
+                              (strerror errno)))))))
           ((streamp object)
            (ecase direction
              (:input
@@ -1223,13 +1197,16 @@ Users Manual for details about the PROCESS structure.
                   (when fd
                     (return (values fd stream))))
                 (multiple-value-bind (read-fd write-fd)
-                    (sb-unix:unix-pipe)
+                    #-win32 (sb-unix:unix-pipe)
+                    #+win32 (sb-win32::make-named-pipe)
                   (unless read-fd
-                    (error "couldn't create pipe: ~S" (strerror write-fd)))
+                    (fail "couldn't create pipe: ~S" (strerror write-fd)))
+                  #+win32
+                  (setf (sb-win32::inheritable-handle-p write-fd) t)
                   (copy-descriptor-to-stream read-fd object cookie
                                              external-format)
                   (push read-fd *close-on-error*)
                   (push write-fd *close-in-parent*)
-                  (return (values write-fd nil)))))
-             (t
-              (error "invalid option to RUN-PROGRAM: ~S" object))))))))
+                  (return (values write-fd nil)))))))
+          (t
+           (fail "invalid option: ~S" object))))))
