@@ -56,7 +56,6 @@
         (oops))))))
 
 (defmacro define-alien-variable (name type &environment env)
-  #!+sb-doc
   "Define NAME as an external alien variable of type TYPE. NAME should
 be a list of a string holding the alien name and a symbol to use as
 the Lisp name. If NAME is just a symbol or string, then the other name
@@ -82,10 +81,10 @@ is guessed from the one supplied."
           (make-heap-alien-info :type type
                                 :alien-name alien-name
                                 :datap t))
-    (setf (info :source-location :variable lisp-name) location)))
+    (setf (info :source-location :variable lisp-name) location)
+    lisp-name))
 
 (defun alien-value (symbol)
-  #!+sb-doc
   "Returns the value of the alien variable bound to SYMBOL. Signals an
 error if SYMBOL is not bound to an alien variable, or if the alien
 variable is undefined."
@@ -93,18 +92,17 @@ variable is undefined."
                    (error 'unbound-variable :name symbol))))
 
 (defmacro extern-alien (name type &environment env)
-  #!+sb-doc
   "Access the alien variable named NAME, assuming it is of type TYPE.
 This is SETFable."
-  (let* ((alien-name (etypecase name
+  (let* ((alien-name (possibly-base-stringize
+                      (etypecase name
                        (symbol (guess-alien-name-from-lisp-name name))
-                       (string name)))
+                       (string name))))
          (alien-type (parse-alien-type type env))
          (datap (not (alien-fun-type-p alien-type))))
     `(%alien-value (foreign-symbol-sap ,alien-name ,datap) 0 ',alien-type)))
 
 (defmacro with-alien (bindings &body body &environment env)
-  #!+sb-doc
   "Establish some local alien variables. Each BINDING is of the form:
      VAR TYPE [ ALLOCATION ] [ INITIAL-VALUE | EXTERNAL-NAME ]
    ALLOCATION should be one of:
@@ -119,117 +117,114 @@ This is SETFable."
   ;;        is allocated at load time, so the same piece of memory is used each time
   ;;        this form executes.
   (/show "entering WITH-ALIEN" bindings)
-  (with-auxiliary-alien-types env
-    (dolist (binding (reverse bindings))
-      (/show binding)
-      (destructuring-bind
-          (symbol type &optional opt1 (opt2 nil opt2p))
-          binding
-        (/show symbol type opt1 opt2)
-        (let* ((alien-type (parse-alien-type type env))
-               (datap (not (alien-fun-type-p alien-type))))
-          (/show alien-type)
-          (multiple-value-bind (allocation initial-value)
-              (if opt2p
-                  (values opt1 opt2)
-                  (case opt1
-                    (:extern
-                     (values opt1 (guess-alien-name-from-lisp-name symbol)))
-                    (:static
-                     (values opt1 nil))
-                    (t
-                     (values :local opt1))))
-            (/show allocation initial-value)
-            (setf body
-                  (ecase allocation
-                    #+nil
-                    (:static
-                     (let ((sap
-                            (make-symbol (concatenate 'string "SAP-FOR-"
-                                                      (symbol-name symbol)))))
-                       `((let ((,sap (load-time-value (%make-alien ...))))
-                           (declare (type system-area-pointer ,sap))
-                           (symbol-macrolet
-                               ((,symbol (sap-alien ,sap ,type)))
-                             ,@(when initial-value
-                                 `((setq ,symbol ,initial-value)))
-                             ,@body)))))
-                    (:extern
-                     (/show0 ":EXTERN case")
-                     `((symbol-macrolet
-                           ((,symbol
-                              (%alien-value
-                               (foreign-symbol-sap ,initial-value ,datap) 0 ,alien-type)))
-                         ,@body)))
-                    (:local
-                     (/show0 ":LOCAL case")
-                     (let* ((var (sb!xc:gensym "VAR"))
-                            (initval (if initial-value (sb!xc:gensym "INITVAL")))
-                            (info (make-local-alien-info :type alien-type))
-                            (inner-body
-                             `((note-local-alien-type ',info ,var)
-                               (symbol-macrolet ((,symbol (local-alien ',info ,var)))
-                                 ,@(when initial-value
-                                     `((setq ,symbol ,initval)))
-                                 ,@body)))
-                            (body-forms
-                             (if initial-value
-                                 `((let ((,initval ,initial-value))
-                                     ,@inner-body))
-                                 inner-body)))
-                       (/show var initval info)
-                       #!+(or x86 x86-64)
-                       `((let ((,var (make-local-alien ',info)))
-                           ,@body-forms))
-                       ;; FIXME: This version is less efficient then it needs to be, since
-                       ;; it could just save and restore the number-stack pointer once,
-                       ;; instead of doing multiple decrements if there are multiple bindings.
-                       #!-(or x86 x86-64)
-                       `((let ((,var (make-local-alien ',info)))
-                           (multiple-value-prog1
-                               (progn ,@body-forms)
-                             ;; No need for unwind protect here, since
-                             ;; allocation involves modifying NSP, and
-                             ;; NSP is saved and restored during NLX.
-                             ;; And in non-transformed case it
-                             ;; performs finalization.
-                             (dispose-local-alien ',info ,var))))))))))))
-    (/show "revised" body)
-    (verify-local-auxiliaries-okay)
-    (/show0 "back from VERIFY-LOCAL-AUXILIARIES-OK, returning")
-    `(symbol-macrolet ((&auxiliary-type-definitions&
-                        ,(append *new-auxiliary-types*
-                                 (auxiliary-type-definitions env))))
-       #!+(or x86 x86-64)
-       (let ((sb!vm::*alien-stack-pointer* sb!vm::*alien-stack-pointer*))
-         ,@body)
-       #!-(or x86 x86-64)
-       ,@body)))
+  (let (#!+c-stack-is-control-stack
+        bind-alien-stack-pointer)
+    (with-auxiliary-alien-types env
+      (dolist (binding (reverse bindings))
+        (/show binding)
+        (destructuring-bind
+            (symbol type &optional opt1 (opt2 nil opt2p))
+            binding
+          (/show symbol type opt1 opt2)
+          (let* ((alien-type (parse-alien-type type env))
+                 (datap (not (alien-fun-type-p alien-type))))
+            (/show alien-type)
+            (multiple-value-bind (allocation initial-value)
+                (if opt2p
+                    (values opt1 opt2)
+                    (case opt1
+                      (:extern
+                       (values opt1 (guess-alien-name-from-lisp-name symbol)))
+                      (:static
+                       (values opt1 nil))
+                      (t
+                       (values :local opt1))))
+              (/show allocation initial-value)
+              (setf body
+                    (ecase allocation
+                      #+nil
+                      (:static
+                       (let ((sap
+                               (make-symbol (concatenate 'string "SAP-FOR-"
+                                                         (symbol-name symbol)))))
+                         `((let ((,sap (load-time-value (%make-alien ...))))
+                             (declare (type system-area-pointer ,sap))
+                             (symbol-macrolet
+                                 ((,symbol (sap-alien ,sap ,type)))
+                               ,@(when initial-value
+                                   `((setq ,symbol ,initial-value)))
+                               ,@body)))))
+                      (:extern
+                       (/show0 ":EXTERN case")
+                       `((symbol-macrolet
+                             ((,symbol
+                                (%alien-value
+                                 (foreign-symbol-sap ,initial-value ,datap) 0 ,alien-type)))
+                           ,@body)))
+                      (:local
+                       (/show0 ":LOCAL case")
+                       (let* ((var (sb!xc:gensym "VAR"))
+                              (initval (if initial-value (sb!xc:gensym "INITVAL")))
+                              (info (make-local-alien-info :type alien-type))
+                              (inner-body
+                                `((note-local-alien-type ',info ,var)
+                                  (symbol-macrolet ((,symbol (local-alien ',info ,var)))
+                                    ,@(when initial-value
+                                        `((setq ,symbol ,initval)))
+                                    ,@body)))
+                              (body-forms
+                                (if initial-value
+                                    `((let ((,initval ,initial-value))
+                                        ,@inner-body))
+                                    inner-body)))
+                         (/show var initval info)
+                         #!+c-stack-is-control-stack
+                         (progn (setf bind-alien-stack-pointer t)
+                                `((let ((,var (make-local-alien ',info)))
+                                    ,@body-forms)))
+                         ;; FIXME: This version is less efficient then it needs to be, since
+                         ;; it could just save and restore the number-stack pointer once,
+                         ;; instead of doing multiple decrements if there are multiple bindings.
+                         #!-c-stack-is-control-stack
+                         `((let ((,var (make-local-alien ',info)))
+                             (multiple-value-prog1
+                                 (progn ,@body-forms)
+                               ;; No need for unwind protect here, since
+                               ;; allocation involves modifying NSP, and
+                               ;; NSP is saved and restored during NLX.
+                               ;; And in non-transformed case it
+                               ;; performs finalization.
+                               (dispose-local-alien ',info ,var))))))))))))
+      (/show "revised" body)
+      (verify-local-auxiliaries-okay)
+      (/show0 "back from VERIFY-LOCAL-AUXILIARIES-OK, returning")
+      `(symbol-macrolet ((&auxiliary-type-definitions&
+                           ,(append *new-auxiliary-types*
+                                    (auxiliary-type-definitions env))))
+         ,@(cond
+             #!+c-stack-is-control-stack
+             (bind-alien-stack-pointer
+              `((let ((sb!vm::*alien-stack-pointer* sb!vm::*alien-stack-pointer*))
+                  ,@body)))
+             (t
+              body))))))
 
 ;;;; runtime C values that don't correspond directly to Lisp types
 
-;;; Note: The DEFSTRUCT for ALIEN-VALUE lives in a separate file
-;;; 'cause it has to be real early in the cold-load order.
-#!-sb-fluid (declaim (freeze-type alien-value))
-(def!method print-object ((value alien-value) stream)
+(defmethod print-object ((value alien-value) stream)
+  ;; Don't use ":TYPE T" here - TYPE-OF isn't what we want.
   (print-unreadable-object (value stream)
-    ;; See identical kludge in host-alieneval.
-    (let ((sb!pretty:*pprint-quote-with-syntactic-sugar* nil))
-      (declare (special sb!pretty:*pprint-quote-with-syntactic-sugar*))
-      (format stream
-            "~S ~S #X~8,'0X ~S ~S"
+    (format stream "~S ~S #X~8,'0X ~S ~/sb!impl:print-type-specifier/"
             'alien-value
             :sap (sap-int (alien-value-sap value))
-            :type (unparse-alien-type (alien-value-type value))))))
+            :type (unparse-alien-type (alien-value-type value)))))
 
 #!-sb-fluid (declaim (inline null-alien))
 (defun null-alien (x)
-  #!+sb-doc
   "Return true if X (which must be an ALIEN pointer) is null, false otherwise."
   (zerop (sap-int (alien-sap x))))
 
 (defmacro sap-alien (sap type &environment env)
-  #!+sb-doc
   "Convert the system area pointer SAP to an ALIEN of the specified TYPE (not
    evaluated.) TYPE must be pointer-like."
   (let ((alien-type (parse-alien-type type env)))
@@ -237,13 +232,7 @@ This is SETFable."
         `(%sap-alien ,sap ',alien-type)
         (error "cannot make an alien of type ~S out of a SAP" type))))
 
-(defun %sap-alien (sap type)
-  (declare (type system-area-pointer sap)
-           (type alien-type type))
-  (make-alien-value :sap sap :type type))
-
 (defun alien-sap (alien)
-  #!+sb-doc
   "Return a System-Area-Pointer pointing to Alien's data."
   (declare (type alien-value alien))
   (alien-value-sap alien))
@@ -251,7 +240,6 @@ This is SETFable."
 ;;;; allocation/deallocation of heap aliens
 
 (defmacro make-alien (type &optional size &environment env)
-  #!+sb-doc
   "Allocate an alien of type TYPE in foreign heap, and return an alien
 pointer to it. The allocated memory is not initialized, and may
 contain garbage. The memory is allocated using malloc(3), so it can be
@@ -340,16 +328,38 @@ Examples:
         (malloc-error bytes (get-errno))
         sap)))
 
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defvar *saved-fp-and-pcs* nil)
+  ;; Can't use DECLAIM since always-bound is a non-standard declaration
+  (sb!xc:proclaim '(sb!ext:always-bound *saved-fp-and-pcs*)))
+
+#!+c-stack-is-control-stack
+(declaim (inline invoke-with-saved-fp-and-pc))
+;;; On :c-stack-is-control-stack platforms, this DEFUN must appear prior to the
+;;; first cross-compile-time use of ALIEN-FUNCALL, the transform of which is
+;;; an invocation of INVOKE-WITH-SAVED-FP-AND-PC, which should be inlined.
+;;; Makes no sense when compiling for the host.
+#!+(and c-stack-is-control-stack (host-feature sb-xc))
+(defun invoke-with-saved-fp-and-pc (fn)
+  (declare #-sb-xc-host (muffle-conditions compiler-note)
+           (optimize (speed 3)))
+  (dx-let ((fp-and-pc (make-array 2 :element-type 'word)))
+    (setf (aref fp-and-pc 0) (sb!kernel:get-lisp-obj-address
+                              (sb!kernel:%caller-frame))
+          (aref fp-and-pc 1) (sap-int (sb!kernel:%caller-pc)))
+    (dx-let ((*saved-fp-and-pcs* (cons fp-and-pc *saved-fp-and-pcs*)))
+      (funcall fn))))
+
 #!-sb-fluid (declaim (inline free-alien))
 (defun free-alien (alien)
-  #!+sb-doc
   "Dispose of the storage pointed to by ALIEN. The ALIEN must have been
 allocated by MAKE-ALIEN, MAKE-ALIEN-STRING or malloc(3)."
   (alien-funcall (extern-alien "free" (function (values) system-area-pointer))
                  (alien-sap alien))
   nil)
 
-(declaim (type (sfunction * system-area-pointer) %make-alien-string))
+(declaim (type (function * (values system-area-pointer index))
+               %make-alien-string))
 (defun %make-alien-string (string &key (start 0) end
                                        (external-format :default)
                                        (null-terminate t))
@@ -363,13 +373,12 @@ allocated by MAKE-ALIEN, MAKE-ALIEN-STRING or malloc(3)."
          (count (length octets))
          (buf (%make-alien count)))
     (sb!kernel:copy-ub8-to-system-area octets 0 buf 0 count)
-    buf))
+    (values buf count)))
 
 (defun make-alien-string (string &rest rest
                                  &key (start 0) end
                                       (external-format :default)
                                       (null-terminate t))
-  #!+sb-doc
   "Copy part of STRING delimited by START and END into freshly
 allocated foreign memory, freeable using free(3) or FREE-ALIEN.
 Returns the allocated string as a (* CHAR) alien, and the number of
@@ -402,7 +411,6 @@ null byte."
 ;;; Extract the value from the named slot from the record ALIEN. If
 ;;; ALIEN is actually a pointer, then DEREF it first.
 (defun slot (alien slot)
-  #!+sb-doc
   "Extract SLOT from the Alien STRUCT or UNION ALIEN. May be set with SETF."
   (declare (type alien-value alien)
            (type symbol slot)
@@ -495,7 +503,6 @@ null byte."
 
 ;;; Dereference the alien and return the results.
 (defun deref (alien &rest indices)
-  #!+sb-doc
   "Dereference an Alien pointer or array. If an array, the indices are used
    as the indices of the array element to access. If a pointer, one index can
    optionally be specified, giving the equivalent of C pointer arithmetic."
@@ -614,7 +621,6 @@ null byte."
 ;;;; the CAST macro
 
 (defmacro cast (alien type &environment env)
-  #!+sb-doc
   "Convert ALIEN to an Alien of the specified TYPE (not evaluated.)  Both types
    must be Alien array, pointer or function types."
   `(%cast ,alien ',(parse-alien-type type env)))
@@ -638,7 +644,6 @@ null byte."
 ;;;; the ALIEN-SIZE macro
 
 (defmacro alien-size (type &optional (units :bits) &environment env)
-  #!+sb-doc
   "Return the size of the alien type TYPE. UNITS specifies the units to
    use and can be either :BITS, :BYTES, or :WORDS."
   (let* ((alien-type (parse-alien-type type env))
@@ -654,8 +659,22 @@ null byte."
 
 ;;;; NATURALIZE, DEPORT, EXTRACT-ALIEN-VALUE, DEPOSIT-ALIEN-VALUE
 
-(defun coerce-to-interpreted-function (lambda-form)
-  (let (#!+sb-eval
+;;; There is little cost to making an interpreted function,
+;;; however it is even better if we can share the function object,
+;;; especially with sb-fasteval which avoids work on repeated invocations.
+;;; sb-eval doesn't optimize its IR in the same way,
+;;; but this is still a boon from a memory consumption stance.
+;;;
+;;; GLOBALDB-SXHASHOID serves as a nice hash function for this purpose anyway.
+;;; Arguably we could key the cache off the internalized alien-type object
+;;; which induced creation of an interpreted lambda, rather than the s-expr,
+;;; but we'd need to get the TYPE and the method {NATURALIZE, DEPORT, etc} here,
+;;; so it would be a more invasive change.
+;;;
+(defun-cached (coerce-to-interpreted-function
+               :hash-bits 8 :hash-function #'globaldb-sxhashoid)
+    ((lambda-form equal))
+  (let (#!+(or sb-eval sb-fasteval)
         (*evaluator-mode* :interpret))
     (coerce lambda-form 'function)))
 
@@ -691,7 +710,6 @@ null byte."
 ;;;; ALIEN-FUNCALL, DEFINE-ALIEN-ROUTINE
 
 (defun alien-funcall (alien &rest args)
-  #!+sb-doc
   "Call the foreign function ALIEN with the specified arguments. ALIEN's
 type specifies the argument and result types."
   (declare (type alien-value alien))
@@ -724,7 +742,6 @@ type specifies the argument and result types."
 (defmacro define-alien-routine (name result-type
                                      &rest args
                                      &environment lexenv)
-  #!+sb-doc
   "DEFINE-ALIEN-ROUTINE Name Result-Type {(Arg-Name Arg-Type [Style])}*
 
 Define a foreign interface function for the routine with the specified NAME.
@@ -831,7 +848,6 @@ way that the argument is passed.
                              ,@(results))))))))))
 
 (defun alien-typep (object type)
-  #!+sb-doc
   "Return T iff OBJECT is an alien of type TYPE."
   (let ((lisp-rep-type (compute-lisp-rep-type type)))
     (if lisp-rep-type
@@ -848,7 +864,6 @@ way that the argument is passed.
 ;;;; See "Foreign Linkage / Callbacks" in the SBCL Internals manual.
 
 (defvar *alien-callback-info* nil
-  #!+sb-doc
   "Maps SAPs to corresponding CALLBACK-INFO structures: contains all the
 information we need to manipulate callbacks after their creation. Used for
 changing the lisp-side function they point to, invalidation, etc.")
@@ -865,19 +880,17 @@ changing the lisp-side function they point to, invalidation, etc.")
 (defun alien-callback-info (alien)
   (cdr (assoc (alien-sap alien) *alien-callback-info* :test #'sap=)))
 
-(defvar *alien-callbacks* (make-hash-table :test #'equal)
-  #!+sb-doc
+(define-load-time-global *alien-callbacks* (make-hash-table :test #'equal)
   "Cache of existing callback SAPs, indexed with (SPECIFER . FUNCTION). Used for
 memoization: we don't create new callbacks if one pointing to the correct
 function with the same specifier already exists.")
 
-(defvar *alien-callback-wrappers* (make-hash-table :test #'equal)
-  #!+sb-doc
+(define-load-time-global *alien-callback-wrappers* (make-hash-table :test #'equal)
   "Cache of existing lisp wrappers, indexed with SPECIFER. Used for memoization:
 we don't create new wrappers if one for the same specifier already exists.")
 
-(defvar *alien-callback-trampolines* (make-array 32 :fill-pointer 0 :adjustable t)
-  #!+sb-doc
+(define-load-time-global *alien-callback-trampolines*
+    (make-array 32 :fill-pointer 0 :adjustable t)
   "Lisp trampoline store: assembler wrappers contain indexes to this, and
 ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
 
@@ -938,10 +951,20 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
              ,(loop
                  with offset = 0
                  for spec in argument-specs
+                 ;; KLUDGE: At least one platform requires additional
+                 ;; alignment beyond a single machine word for certain
+                 ;; arguments.  Accept an additional delta (for the
+                 ;; alignment) to apply to subsequent arguments to
+                 ;; account for the alignment gaps as a secondary
+                 ;; value, so that we don't have to update unaffected
+                 ;; backends.
+                 for (accessor-form alignment)
+                   = (multiple-value-list
+                      (alien-callback-accessor-form spec 'args-sap offset))
                  collect `(,(pop argument-names) ,spec
-                            :local ,(alien-callback-accessor-form
-                                     spec 'args-sap offset))
-                 do (incf offset (alien-callback-argument-bytes spec env)))
+                            :local ,accessor-form)
+                 do (incf offset (+ (alien-callback-argument-bytes spec env)
+                                    (or alignment 0))))
            ,(flet ((store (spec real-type)
                           (if spec
                               `(setf (deref (sap-alien res-sap (* ,spec)))
@@ -1014,15 +1037,10 @@ ENTER-ALIEN-CALLBACK pulls the corresponding trampoline out and calls it.")
            return
            arguments))
 
-;;; To ensure that callback wrapper functions continue working even
-;;; if #'ENTER-ALIEN-CALLBACK moves in memory, access to it is indirected
-;;; through the *ENTER-ALIEN-CALLBACK* static symbol. -- JES, 2006-01-01
-(defvar *enter-alien-callback* #'enter-alien-callback)
-
 ;;;; interface (not public, yet) for alien callbacks
 
+(let ()
 (defmacro alien-callback (specifier function &environment env)
-  #!+sb-doc
   "Returns an alien-value with of alien ftype SPECIFIER, that can be passed to
 an alien function as a pointer to the FUNCTION. If a callback for the given
 SPECIFIER and FUNCTION already exists, it is returned instead of consing a new
@@ -1040,10 +1058,9 @@ one."
                                               ',(alien-callback-lisp-wrapper-lambda
                                                  specifier result-type argument-types env))))
                            ,call-type)
-      ',(parse-alien-type specifier env))))
+      ',(parse-alien-type specifier env)))))
 
 (defun alien-callback-p (alien)
-  #!+sb-doc
   "Returns true if the alien is associated with a lisp-side callback,
 and a secondary return value of true if the callback is still valid."
   (let ((info (alien-callback-info alien)))
@@ -1051,14 +1068,12 @@ and a secondary return value of true if the callback is still valid."
       (values t (and (callback-info-function info) t)))))
 
 (defun alien-callback-function (alien)
-  #!+sb-doc
   "Returns the lisp function designator associated with the callback."
   (let ((info (alien-callback-info alien)))
     (when info
       (callback-info-function info))))
 
 (defun (setf alien-callback-function) (function alien)
-  #!+sb-doc
   "Changes the lisp function designated by the callback."
   (let ((info (alien-callback-info alien)))
     (unless info
@@ -1075,7 +1090,6 @@ and a secondary return value of true if the callback is still valid."
     function))
 
 (defun invalidate-alien-callback (alien)
-  #!+sb-doc
   "Invalidates the callback designated by the alien, if any, allowing the
 associated lisp function to be GC'd, and causing further calls to the same
 callback signal an error."
@@ -1097,6 +1111,7 @@ callback signal an error."
 ;;;
 ;;; For lambdas that result in simple-funs we get the callback from
 ;;; the cache on subsequent calls.
+(let ()
 (defmacro alien-lambda (result-type typed-lambda-list &body forms)
   (multiple-value-bind (specifier lambda-list)
       (parse-callback-specification result-type typed-lambda-list)
@@ -1107,7 +1122,6 @@ callback signal an error."
 ;;; the FDEFINITION should invalidate the callback, and redefining the
 ;;; callback should change existing callbacks to point to the new defintion.
 (defmacro define-alien-callback (name result-type typed-lambda-list &body forms)
-  #!+sb-doc
   "Defines #'NAME as a function with the given body and lambda-list, and NAME as
 the alien callback for that function with the given alien type."
   (declare (symbol name))
@@ -1116,3 +1130,4 @@ the alien callback for that function with the given alien type."
     `(progn
        (defun ,name ,lambda-list ,@forms)
        (defparameter ,name (alien-callback ,specifier #',name)))))
+)
