@@ -19,28 +19,33 @@
 
 ;;;; variables initialization and shutdown sequences
 
-;; (Most of the save-a-core functionality is defined later, in its
-;; own file, but we'd like to have these symbols declared special
-;; and initialized ASAP.)
+;;; (Most of the save-a-core functionality is defined later, in its
+;;; own file, but we'd like to have these symbols declared special and
+;;; initialized ASAP.)
+
+(declaim (type list *save-hooks* *init-hooks* *exit-hooks*))
+
 (defvar *save-hooks* nil
-  #!+sb-doc
-  "This is a list of functions which are called in an unspecified
-order before creating a saved core image. Unused by SBCL itself:
-reserved for user and applications.")
+  "A list of function designators which are called in an unspecified
+order before creating a saved core image.
+
+Unused by SBCL itself: reserved for user and applications.")
 
 (defvar *init-hooks* nil
-  #!+sb-doc
-  "This is a list of functions which are called in an unspecified
+  "A list of function designators which are called in an unspecified
 order when a saved core image starts up, after the system itself has
-been initialized. Unused by SBCL itself: reserved for user and
-applications.")
+been initialized.
+
+Unused by SBCL itself: reserved for user and applications.")
 
 (defvar *exit-hooks* nil
-  #!+sb-doc
-  "This is a list of functions which are called in an unspecified
-order when SBCL process exits. Unused by SBCL itself: reserved for
-user and applications. Using (SB-EXT:EXIT :ABORT T), or calling
-exit(3) directly will circumvent these hooks.")
+  "A list of function designators which are called in an unspecified
+order when SBCL process exits.
+
+Unused by SBCL itself: reserved for user and applications.
+
+Using (SB-EXT:EXIT :ABORT T), or calling exit(3) directly circumvents
+these hooks.")
 
 
 ;;; Binary search for simple vectors
@@ -79,15 +84,6 @@ exit(3) directly will circumvent these hooks.")
     (recurse 0 (truncate (length vector) 2))))
 
 
-;;; like LISTEN, but any whitespace in the input stream will be flushed
-(defun listen-skip-whitespace (&optional (stream *standard-input*))
-  (do ((char (read-char-no-hang stream nil nil nil)
-             (read-char-no-hang stream nil nil nil)))
-      ((null char) nil)
-    (cond ((not (whitespace[1]p char))
-           (unread-char char stream)
-           (return t)))))
-
 ;;;; helpers for C library calls
 
 ;;; Signal a SIMPLE-CONDITION/ERROR condition associated with an ANSI C
