@@ -14,6 +14,15 @@
 
 ;;;; utilities
 
+(defun hash-table-weakness (ht)
+  "Return the WEAKNESS of HASH-TABLE which is one of NIL, :KEY,
+:VALUE, :KEY-AND-VALUE, :KEY-OR-VALUE."
+  (aref weak-hash-table-kinds (hash-table-%weakness ht)))
+
+(declaim (inline hash-table-weak-p))
+(defun hash-table-weak-p (ht)
+  (not (zerop (hash-table-%weakness ht))))
+
 ;;; Code for detecting concurrent accesses to the same table from
 ;;; multiple threads. Only compiled in when the :SB-HASH-TABLE-DEBUG
 ;;; feature is enabled. The main reason for the existence of this code
@@ -381,7 +390,7 @@ Examples:
                    rehash-threshold
                    size
                    kv-vector
-                   weakness
+                   (position weakness weak-hash-table-kinds :test #'eq)
                    index-vector
                    next-vector
                    (unless (eq test 'eq)
@@ -424,10 +433,6 @@ Examples:
 (setf (fdocumentation 'hash-table-test 'function)
       "Return the test HASH-TABLE was created with.")
 
-(setf (fdocumentation 'hash-table-weakness 'function)
-      "Return the WEAKNESS of HASH-TABLE which is one of NIL, :KEY,
-:VALUE, :KEY-AND-VALUE, :KEY-OR-VALUE.")
-
 ;;; Called when we detect circular chains in a hash-table.
 (defun signal-corrupt-hash-table (hash-table)
   (error "Corrupt NEXT-chain in ~A. This is probably caused by ~
@@ -476,7 +481,7 @@ multiple threads accessing the same hash-table without locking."
     (set-header-data old-kv-vector sb!vm:vector-normal-subtype)
 
     ;; Non-empty weak hash tables always need GC support.
-    (when (and (hash-table-weakness table) (plusp (hash-table-count table)))
+    (when (and (hash-table-weak-p table) (plusp (hash-table-count table)))
       (set-header-data new-kv-vector sb!vm:vector-valid-hashing-subtype))
 
     ;; FIXME: here and in several other places in the hash table code,
@@ -503,8 +508,7 @@ multiple threads accessing the same hash-table without locking."
       (declare (type index/2 i))
       (let ((key (aref new-kv-vector (* 2 i)))
             (value (aref new-kv-vector (1+ (* 2 i)))))
-        (cond ((and (eq key +empty-ht-slot+)
-                    (eq value +empty-ht-slot+))
+        (cond ((and (empty-ht-slot-p key) (empty-ht-slot-p value))
                ;; Slot is empty, push it onto the free list.
                (setf (aref new-next-vector i)
                      (hash-table-next-free-kv table))
@@ -559,7 +563,7 @@ multiple threads accessing the same hash-table without locking."
     (declare (type index size length))
 
     ;; Non-empty weak hash tables always need GC support.
-    (unless (and (hash-table-weakness table) (plusp (hash-table-count table)))
+    (unless (and (hash-table-weak-p table) (plusp (hash-table-count table)))
       ;; Disable GC tricks, they will be re-enabled during the re-hash
       ;; if necessary.
       (set-header-data kv-vector sb!vm:vector-normal-subtype))
@@ -575,8 +579,7 @@ multiple threads accessing the same hash-table without locking."
       (declare (type index/2 i))
       (let ((key (aref kv-vector (* 2 i)))
             (value (aref kv-vector (1+ (* 2 i)))))
-        (cond ((and (eq key +empty-ht-slot+)
-                    (eq value +empty-ht-slot+))
+        (cond ((and (empty-ht-slot-p key) (empty-ht-slot-p value))
                ;; Slot is empty, push it onto free list.
                (setf (aref next-vector i) (hash-table-next-free-kv table))
                (setf (hash-table-next-free-kv table) i))
@@ -609,7 +612,7 @@ multiple threads accessing the same hash-table without locking."
 
 (declaim (inline maybe-rehash))
 (defun maybe-rehash (hash-table ensure-free-slot-p)
-  (when (hash-table-weakness hash-table)
+  (when (hash-table-weak-p hash-table)
     (aver *gc-inhibit*))
   (flet ((rehash-p ()
            (and ensure-free-slot-p
@@ -636,7 +639,7 @@ multiple threads accessing the same hash-table without locking."
 
 (declaim (inline update-hash-table-cache))
 (defun update-hash-table-cache (hash-table index)
-  (unless (hash-table-weakness hash-table)
+  (unless (hash-table-weak-p hash-table)
     (setf (hash-table-cache hash-table) index)))
 
 (defmacro with-hash-table-locks ((hash-table
@@ -649,7 +652,7 @@ multiple threads accessing the same hash-table without locking."
               (with-concurrent-access-check ,hash-table ,operation
                 (locally (declare (inline ,@inline))
                   ,@body))))
-       (if (hash-table-weakness ,hash-table)
+       (if (hash-table-weak-p ,hash-table)
            (sb!thread::with-recursive-system-lock
                ((hash-table-lock ,hash-table) :without-gcing t)
              (,body-fun))
@@ -779,7 +782,7 @@ if there is no such entry. Entries can be added using SETF."
            (hash-vector (hash-table-hash-vector hash-table))
            (test-fun (hash-table-test-fun hash-table)))
       (declare (type index index next))
-      (when (hash-table-weakness hash-table)
+      (when (hash-table-weak-p hash-table)
         (set-header-data kv-vector sb!vm:vector-valid-hashing-subtype))
       (cond ((or eq-based (not hash-vector))
              (when eq-based
