@@ -48,7 +48,7 @@
 ;;;; sufficient, though interaction between parallel intern and use-package
 ;;;; needs to be considered with some care.
 
-(defvar *package-graph-lock*)
+(define-load-time-global *package-graph-lock* nil)
 
 (defun call-with-package-graph (function)
   (declare (function function))
@@ -58,7 +58,7 @@
     (funcall function)))
 
 ;;; a map from package names to packages
-(defvar *package-names*)
+(define-load-time-global *package-names* nil)
 (declaim (type hash-table *package-names*))
 
 (defmacro with-package-names ((names &key) &body body)
@@ -541,6 +541,12 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
                               (when (boundp '*package*)
                                 *package*)))
 
+;;; Perform (GETHASH NAME TABLE) and then unwrap the value if it is a list.
+;;; List vs nonlist disambiguates a nickname from the primary name.
+(defun %get-package (name table)
+  (let ((found (gethash name table)))
+    (if (listp found) (car found) found)))
+
 ;;; This is undocumented and unexported for now, but the idea is that by
 ;;; making this a generic function then packages with custom package classes
 ;;; could hook into this to provide their own resolution.
@@ -556,7 +562,7 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
                                (package-%local-nicknames base)))
                   (nicknamed (when nicknames
                                (cdr (assoc string nicknames :test #'string=))))
-                  (packageoid (or nicknamed (gethash string *package-names*))))
+                  (packageoid (or nicknamed (%get-package string *package-names*))))
              (if (and (null packageoid)
                       ;; FIXME: should never need 'debootstrap' hack
                       (let ((mismatch (mismatch "SB!" string)))
@@ -565,7 +571,7 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
                      (signal 'bootstrap-package-not-found :name string)
                    (debootstrap-package ()
                      (if (string= string "SB!XC")
-                         (find-package "COMMON-LISP")
+                         *cl-package*
                          (find-package
                           (substitute #\- #\! string :count 1)))))
                  packageoid))))
@@ -697,13 +703,13 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 ;;; Enter any new NICKNAMES for PACKAGE into *PACKAGE-NAMES*. If there is a
 ;;; conflict then give the user a chance to do something about it. Caller is
 ;;; responsible for having acquired the mutex via WITH-PACKAGES.
-(defun %enter-new-nicknames (package nicknames)
+(defun %enter-new-nicknames (package nicknames &aux (val (list package)))
   (declare (type list nicknames))
   (dolist (n nicknames)
     (let ((found (with-package-names (names)
-                    (or (gethash (the simple-string n) names)
+                    (or (%get-package (the simple-string n) names)
                         (progn
-                          (setf (gethash n names) package)
+                          (setf (gethash n names) val)
                           (push n (package-%nicknames package))
                           package)))))
       (cond ((eq found package))
@@ -897,7 +903,8 @@ implementation it is ~S." *!default-package-use-list*)
     (with-package-names (names)
       (maphash (lambda (k v)
                  (declare (ignore k))
-                 (pushnew v res :test #'eq))
+                 (unless (listp v)
+                   (push v res)))
                names))
     res))
 
@@ -1014,7 +1021,7 @@ implementation it is ~S." *!default-package-use-list*)
   (values nil nil))
 
 ;;; Similar to FIND-SYMBOL, but only looks for an external symbol.
-;;; Return the symbol and T if found, otherwise two NILs.
+;;; Return the symbol if found, otherwise 0.
 ;;; This is used for fast name-conflict checking in this file and symbol
 ;;; printing in the printer.
 ;;; An optimization is possible here: by accepting either a string or symbol,
@@ -1025,8 +1032,8 @@ implementation it is ~S." *!default-package-use-list*)
          (hash (compute-symbol-hash string length)))
     (declare (type index length) (type hash hash))
     (with-symbol ((symbol) (package-external-symbols package) string length hash)
-      (return-from find-external-symbol (values symbol t))))
-  (values nil nil))
+      (return-from find-external-symbol symbol)))
+  0)
 
 (define-condition name-conflict (reference-condition package-error)
   ((function :initarg :function :reader name-conflict-function)
@@ -1167,10 +1174,8 @@ uninterned."
         (when (member symbol shadowing-symbols)
           (let ((cset ()))
             (dolist (p (package-%use-list package))
-              (multiple-value-bind (s w) (find-external-symbol name p)
-                ;; S should be derived as SYMBOL so that PUSHNEW can assume #'EQ
-                ;; as the test, but it's not happening, so restate the obvious.
-                (when w (pushnew s cset :test #'eq))))
+              (let ((s (find-external-symbol name p)))
+                (unless (eql s 0) (pushnew s cset :test #'eq))))
             (when (cdr cset)
               (apply #'name-conflict package 'unintern symbol cset)
               (return-from unintern t)))
@@ -1214,10 +1219,9 @@ uninterned."
           (syms ()))
       ;; Punt any symbols that are already external.
       (dolist (sym symbols)
-        (multiple-value-bind (s found)
-            (find-external-symbol (symbol-name sym) package)
-          (unless (or (and found (eq s sym)) (member sym syms))
-            (push sym syms))))
+        (let ((s (find-external-symbol (symbol-name sym) package)))
+          (unless (eq s sym)
+            (pushnew sym syms :test #'eq))))
       (with-single-package-locked-error ()
         (when syms
           (assert-package-unlocked package "exporting symbol~P ~{~A~^, ~}"
@@ -1412,17 +1416,15 @@ PACKAGE."
                            (incf res (package-external-symbol-count p)))))
                     (package-external-symbol-count pkg))
                  (do-symbols (sym package)
-                   (multiple-value-bind (s w)
-                       (find-external-symbol (symbol-name sym) pkg)
-                     (when (and w
+                   (let ((s (find-external-symbol (symbol-name sym) pkg)))
+                     (when (and (not (eql s 0))
                                 (not (eq s sym))
                                 (not (member sym shadowing-symbols)))
                        (name-conflict package 'use-package pkg sym s))))
                  (dolist (p use-list)
                    (do-external-symbols (sym p)
-                     (multiple-value-bind (s w)
-                         (find-external-symbol (symbol-name sym) pkg)
-                       (when (and w
+                     (let ((s (find-external-symbol (symbol-name sym) pkg)))
+                       (when (and (not (eql s 0))
                                   (not (eq s sym))
                                   (not (member
                                         (find-symbol (symbol-name sym) package)
@@ -1472,8 +1474,9 @@ PACKAGE."
     (with-package-names (names)
       (maphash (lambda (k v)
                  (declare (ignore k))
-                 (multiple-value-bind (s w) (find-symbol string v)
-                   (when w (pushnew s res))))
+                 (unless (listp v) ; ignore nickname entries
+                   (multiple-value-bind (s w) (find-symbol string v)
+                     (when w (pushnew s res)))))
                names))
     res))
 
@@ -1548,14 +1551,14 @@ PACKAGE."
                                (length (the simple-vector input)))))
                    (dovector (symbol input table)
                      (add-symbol table symbol)))))
-          (setf (package-external-symbols pkg) (!make-table (car symbols))
-                (package-internal-symbols pkg) (!make-table (cdr symbols))))
+          (setf (package-external-symbols pkg) (!make-table (first symbols))
+                (package-internal-symbols pkg) (!make-table (second symbols))))
         (setf (package-%local-nicknames pkg) nil
               (package-%locally-nicknamed-by pkg) nil
               (package-source-location pkg) nil
               (gethash (package-%name pkg) names) pkg)
         (dolist (nick (package-%nicknames pkg))
-          (setf (gethash nick names) pkg))
+          (setf (gethash nick names) (list pkg)))
         #!+sb-package-locks
         (setf (package-lock pkg) nil
               (package-%implementation-packages pkg) nil))))
