@@ -486,6 +486,15 @@
                                    (sxhash q)
                                    (sxhash-double-float
                                     (coerce key 'double-float)))))
+                            ((float-infinity-p key)
+                             ;; {single,double}-float infinities are EQUALP
+                             (if (minusp key)
+                                 (load-time-value
+                                  (sxhash (symbol-value 'sb!ext:single-float-negative-infinity))
+                                  t)
+                                 (load-time-value
+                                  (sxhash (symbol-value 'sb!ext:single-float-positive-infinity))
+                                  t)))
                             (t
                              (multiple-value-bind (q r) (floor key)
                                (if (zerop (the ,type r))
