@@ -14,7 +14,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(*condition-name-vec* conditional-opcode emit-word
+  (import '(conditional-opcode emit-word
             composite-immediate-instruction encodable-immediate
             lsl lsr asr ror cpsr @) 'sb!vm)
   ;; Imports from SB-VM into this package
@@ -24,7 +24,7 @@
 (setf *disassem-inst-alignment-bytes* 4)
 
 
-(defparameter *conditions*
+(defconstant-eqx +conditions+
   '((:eq . 0)
     (:ne . 1)
     (:cs . 2) (:hs . 2)
@@ -39,13 +39,14 @@
     (:lt . 11)
     (:gt . 12)
     (:le . 13)
-    (:al . 14)))
-(defparameter *condition-name-vec*
-  (let ((vec (make-array 16 :initial-element nil)))
-    (dolist (cond *conditions*)
-      (when (null (aref vec (cdr cond)))
-        (setf (aref vec (cdr cond)) (car cond))))
-    vec))
+    (:al . 14))
+  #'equal)
+(defconstant-eqx sb!vm::+condition-name-vec+
+  #.(let ((vec (make-array 16 :initial-element nil)))
+      (dolist (cond +conditions+ vec)
+        (when (null (aref vec (cdr cond)))
+          (setf (aref vec (cdr cond)) (car cond)))))
+  #'equalp)
 
 ;;; Set assembler parameters. (In CMU CL, this was done with
 ;;; a call to a macro DEF-ASSEMBLER-PARAMS.)
@@ -53,7 +54,7 @@
   (setf sb!assem:*assem-scheduler-p* nil))
 
 (defun conditional-opcode (condition)
-  (cdr (assoc condition *conditions* :test #'eq)))
+  (cdr (assoc condition +conditions+ :test #'eq)))
 
 ;;;; disassembler field definitions
 
@@ -354,7 +355,7 @@
   (let ((internal-emitter (gensym)))
     `(flet ((,internal-emitter ,arglist
               ,@body))
-       (if (assoc (car ,argvar) *conditions*)
+       (if (assoc (car ,argvar) +conditions+)
            (apply #',internal-emitter ,argvar)
            (apply #',internal-emitter :al ,argvar)))))
 
