@@ -381,8 +381,10 @@
            (if (plusp amount)
                (inst lsl result number amount)
                (inst asr result number (- amount))))
+          ((= amount 64)
+           (inst mov result 0))
           (t
-           (inst mov result 0)))))
+           (inst asr result number 63)))))
 
 (define-vop (fast-ash/signed/unsigned)
   (:note "inline ASH")
@@ -629,7 +631,7 @@
 (defun %%ldb (integer size posn)
   (%ldb size posn integer))
 
-(defun %%dpb (newbyte integer size posn)
+(defun %%dpb (newbyte size posn integer)
   (%dpb newbyte size posn integer))
 
 (define-vop (ldb-c/fixnum)
@@ -912,21 +914,22 @@
 (define-vop (fast-logbitp-c/fixnum fast-conditional-c/fixnum)
   (:translate %logbitp)
   (:conditional :ne)
-  (:arg-types tagged-num (:constant (integer 0 63)))
+  (:arg-types tagged-num (:constant (mod #.n-word-bits)))
   (:generator 4
-    (inst tst x (ash 1 (+ y n-fixnum-tag-bits)))))
+    (inst tst x (ash 1 (min (+ y n-fixnum-tag-bits)
+                            (1- n-word-bits))))))
 
 (define-vop (fast-logbitp-c/signed fast-conditional-c/signed)
   (:translate %logbitp)
   (:conditional :ne)
-  (:arg-types signed-num (:constant (integer 0 63)))
+  (:arg-types signed-num (:constant (mod #.n-word-bits)))
   (:generator 5
     (inst tst x (ash 1 y))))
 
 (define-vop (fast-logbitp-c/unsigned fast-conditional-c/unsigned)
   (:translate %logbitp)
   (:conditional :ne)
-  (:arg-types unsigned-num (:constant (integer 0 63)))
+  (:arg-types unsigned-num (:constant (mod #.n-word-bits)))
   (:generator 5
     (inst tst x (ash 1 y))))
 
