@@ -255,10 +255,13 @@
 
     (when elsewhere-locations
       (dolist (loc (nreverse elsewhere-locations))
-        (push loc locations)
         (dump-location-from-info loc var-locs)))
-    (!make-specialized-array (length byte-buffer) '(unsigned-byte 8)
-                             byte-buffer)))
+    (let ((compressed
+           (lz-compress (coerce byte-buffer
+                                '(simple-array (unsigned-byte 8) (*))))))
+      (logically-readonlyize
+       (!make-specialized-array (length compressed) '(unsigned-byte 8)
+                                compressed)))))
 
 ;;; Return DEBUG-SOURCE structure containing information derived from
 ;;; INFO.
@@ -312,7 +315,8 @@
                     (setf max-negative abs))))
                (t
                 (return-from coerce-to-smallest-eltype
-                  (coerce seq 'simple-vector))))))
+                  (logically-readonlyize
+                   (coerce seq 'simple-vector)))))))
       (if (listp seq)
           (dolist (i seq)
             (incf length)     ; so not to traverse again to compute it
@@ -321,11 +325,19 @@
             (frob i)))
       (if (zerop length)
           #()
-          (!make-specialized-array length
-                                   (smallest-element-type (max max-positive
-                                                               (1- max-negative))
-                                                          (plusp max-negative))
-                                   seq)))))
+          (logically-readonlyize
+           (!make-specialized-array length
+                                    (smallest-element-type (max max-positive
+                                                                (1- max-negative))
+                                                           (plusp max-negative))
+                                    seq))))))
+
+(defun compact-vector (sequence)
+  (cond ((and (= (length sequence) 1)
+              (not (vectorp (elt sequence 0))))
+         (elt sequence 0))
+        (t
+         (coerce-to-smallest-eltype sequence))))
 
 ;;;; variables
 
@@ -354,8 +366,8 @@
 ;;; environment live and is an argument. If a :DEBUG-ENVIRONMENT TN,
 ;;; then we also exclude set variables, since the variable is not
 ;;; guaranteed to be live everywhere in that case.
-(defun dump-1-var (fun var tn id minimal buffer)
-  (declare (type lambda-var var) (type (or tn null) tn) (type index id)
+(defun dump-1-var (fun var tn minimal buffer &optional same-name-p)
+  (declare (type lambda-var var) (type (or tn null) tn)
            (type clambda fun))
   (let* ((name (leaf-debug-name var))
          (save-tn (and tn (tn-save-tn tn)))
@@ -391,9 +403,9 @@
         (:more-count
          (setq flags (logior flags compiled-debug-var-more-count-p)
                more t))))
-    (unless (or more (zerop id) minimal)
-      (setq flags (logior flags compiled-debug-var-id-p)))
-
+    (when (and same-name-p
+               (not (or more minimal)))
+      (setf flags (logior flags compiled-debug-var-same-name-p)))
     #!+64-bit
     (cond (indirect
            (setf (ldb (byte 27 8) flags) (tn-sc-offset tn))
@@ -407,10 +419,15 @@
              (setf (ldb (byte 27 35) flags) (tn-sc-offset save-tn)))))
     (vector-push-extend flags buffer)
     (unless (or minimal
+                same-name-p
                 more) ;; &more vars need no name
-      (vector-push-extend name buffer)
-      (unless (zerop id)
-        (vector-push-extend id buffer)))
+      ;; Dumping uninterned symbols as debug var names is kinda silly.
+      ;; Reconstruction of the name on fasl load produces a new gensym anyway.
+      ;; So rather than waste symbol space, just dump such symbols as strings,
+      ;; and PARSE-COMPILED-DEBUG-VARS can create the interned symbol.
+      ;; This reduces core size by omitting zillions of symbols whose names
+      ;; are spelled the same.
+      (vector-push-extend (if (symbol-package name) name (string name)) buffer))
 
     (cond (indirect
            ;; Indirect variables live in the parent frame, and are
@@ -462,24 +479,20 @@
                         :key (lambda (x)
                                (symbol-name (leaf-debug-name (car x))))))
           (prev-name nil)
-          (id 0)
           (i 0)
           (buffer (make-array 0 :fill-pointer 0 :adjustable t))
           ;; XEPs don't have any useful variables
           (minimal (eq (functional-kind fun) :external)))
-      (declare (type (or simple-string null) prev-name)
-               (type index id i))
+      (declare (type index i))
       (dolist (x sorted)
         (let* ((var (car x))
-               (name (symbol-name (leaf-debug-name var))))
-          (cond ((and prev-name (string= prev-name name))
-                 (incf id))
-                (t
-                 (setq id 0  prev-name name)))
-          (dump-1-var fun var (cdr x) id minimal buffer)
+               (name (leaf-debug-name var)))
+          (dump-1-var fun var (cdr x) minimal buffer
+                      (and prev-name (eq prev-name name)))
+          (setf prev-name name)
           (setf (gethash var var-locs) i)
           (incf i)))
-      (coerce buffer 'simple-vector))))
+      (compact-vector buffer))))
 
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS of
 ;;; FUN, representing the arguments to FUN in minimal variable format.
@@ -487,8 +500,8 @@
   (declare (type clambda fun))
   (let ((buffer (make-array 0 :fill-pointer 0 :adjustable t)))
     (dolist (var (lambda-vars fun))
-      (dump-1-var fun var (leaf-info var) 0 t buffer))
-    (coerce buffer 'simple-vector)))
+      (dump-1-var fun var (leaf-info var) t buffer))
+    (compact-vector buffer)))
 
 ;;; Return VAR's relative position in the function's variables (determined
 ;;; from the VAR-LOCS hashtable).  If VAR is deleted, then return DEBUG-INFO-VAR-DELETED.
@@ -550,7 +563,7 @@
           (dolist (var (lambda-vars fun))
             (res (debug-location-for var var-locs)))))
 
-    (coerce-to-smallest-eltype (res))))
+    (compact-vector (res))))
 
 ;;; Return a vector of SC offsets describing FUN's return locations.
 ;;; (Must be known values return...)
@@ -590,15 +603,15 @@
              #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-return-pc 2env))
              #!-fp-and-pc-standard-save :old-fp
              #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-old-fp 2env))
-             :start-pc (label-position (ir2-physenv-environment-start 2env))
-             :elsewhere-pc (label-position (ir2-physenv-elsewhere-start 2env))
-             :closure-save (when (ir2-physenv-closure-save-tn 2env)
-                             (tn-sc-offset (ir2-physenv-closure-save-tn 2env)))
-             #!+unwind-to-frame-and-call-vop
-             :bsp-save
-             #!+unwind-to-frame-and-call-vop
-             (when (ir2-physenv-bsp-save-tn 2env)
-               (tn-sc-offset (ir2-physenv-bsp-save-tn 2env))))))
+             :encoded-locs
+             (cdf-encode-locs
+              (label-position (ir2-physenv-environment-start 2env))
+              (label-position (ir2-physenv-elsewhere-start 2env))
+              (when (ir2-physenv-closure-save-tn 2env)
+                (tn-sc-offset (ir2-physenv-closure-save-tn 2env)))
+              #!+unwind-to-frame-and-call-vop
+              (when (ir2-physenv-bsp-save-tn 2env)
+                (tn-sc-offset (ir2-physenv-bsp-save-tn 2env)))))))
 
 ;;; Return a complete C-D-F structure for FUN. This involves
 ;;; determining the DEBUG-INFO level and filling in optional slots as
@@ -693,9 +706,7 @@
             (aref (file-info-positions
                    (source-info-file-info *source-info*))
                   component-tlf-num))
-       :contexts
-       (and (plusp (length *contexts*))
-            (coerce *contexts* 'simple-vector))))))
+       :contexts (compact-vector *contexts*)))))
 
 ;;; Write BITS out to BYTE-BUFFER in backend byte order. The length of
 ;;; BITS must be evenly divisible by eight.
