@@ -107,7 +107,8 @@
 
 ;;; This is not FLUSHABLE, since it's required to signal an error if
 ;;; unbound.
-(defknown (symbol-value) (symbol) t ())
+(defknown (symbol-value symeval) (symbol) t ()
+  :derive-type #'symeval-derive-type)
 (defknown about-to-modify-symbol-value (symbol t &optional t t) null
   ())
 ;;; From CLHS, "If the symbol is globally defined as a macro or a
@@ -1591,7 +1592,8 @@
 
 ;;;; miscellaneous extensions
 
-(defknown symbol-global-value (symbol) t ())
+(defknown (symbol-global-value sym-global-val) (symbol) t ()
+  :derive-type #'symeval-derive-type)
 (defknown set-symbol-global-value (symbol t) t ()
   :derive-type #'result-type-last-arg)
 
@@ -1880,7 +1882,9 @@
 ;;;; atomic ops
 (defknown %compare-and-swap-svref (simple-vector index t t) t
     ())
-(defknown %compare-and-swap-symbol-value (symbol t t) t
+(defknown (%compare-and-swap-symbol-value
+           #!+x86-64 %cas-symbol-global-value)
+    (symbol t t) t
     (unwind))
 (defknown (%atomic-dec-symbol-global-value %atomic-inc-symbol-global-value)
     (symbol fixnum) fixnum)
