@@ -12,12 +12,12 @@
 
 (in-package "SB!VM")
 
-(defmacro !configure-dynamic-space-end (&optional default)
+(defun !read-dynamic-space-size ()
   (with-open-file (f "output/dynamic-space-size.txt")
     (let ((line (read-line f)))
       (multiple-value-bind (number end)
           (parse-integer line :junk-allowed t)
-        (if number
+        (when number
             (let* ((ext (subseq line end))
                    (mult (cond ((or (zerop (length ext))
                                     (member ext '("MB" "MIB") :test #'equalp))
@@ -26,12 +26,7 @@
                                 (expt 2 30))
                                (t
                                 (error "Invalid --dynamic-space-size=~A" line)))))
-              `(+ dynamic-space-start ,(* number mult)))
-            (or default
-                `(+ dynamic-space-start
-                    (ecase n-word-bits
-                      (32 (expt 2 29))
-                      (64 (expt 2 30))))))))))
+              (* number mult)))))))
 
 #!+gencgc
 ;; Define START/END constants for GENCGC spaces.
@@ -62,7 +57,7 @@
 (defmacro !gencgc-space-setup
     (small-spaces-start
           &key ((:dynamic-space-start dynamic-space-start*))
-               default-dynamic-space-size
+               ((:default-dynamic-space-size default-dynamic-space-size*))
                #!+immobile-space (immobile-space-size (* 128 1024 1024))
                ;; Smallest os_validate()able alignment; used as safepoint
                ;; page size.  Default suitable for POSIX platforms.
@@ -101,21 +96,22 @@
          (safepoint-page-forms
           (list #!+sb-safepoint
                 `(defconstant gc-safepoint-page-addr ,safepoint-address)))
-         (dynamic-space-start* (or dynamic-space-start* ptr))
-         (optional-dynamic-space-end
-          (when default-dynamic-space-size
-            (list (+ dynamic-space-start* default-dynamic-space-size)))))
+         )
     #+ccl safepoint-address ; workaround for incorrect "Unused" warning
     `(progn
        ,@safepoint-page-forms
        ,@small-space-forms
        #!+immobile-space
        (defconstant immobile-fixedobj-subspace-size (* 24 1024 1024))
-       (defconstant dynamic-space-start ,dynamic-space-start*)
-       (defconstant dynamic-space-end (!configure-dynamic-space-end
-                                       ,@optional-dynamic-space-end)))))
+       (defconstant dynamic-space-start ,(or dynamic-space-start* ptr))
+       (defconstant default-dynamic-space-size
+         (or ,(!read-dynamic-space-size)
+             ,default-dynamic-space-size*
+             (ecase n-word-bits
+               (32 (expt 2 29))
+               (64 (expt 2 30))))))))
 
-(defparameter *c-callable-static-symbols*
+(defconstant-eqx +c-callable-fdefns+
   '(sub-gc
     sb!kernel::post-gc
     internal-error
@@ -132,16 +128,14 @@
     sb!di::handle-single-step-trap
     #!+win32 sb!kernel::handle-win32-exception
     #!+sb-thruption sb!thread::run-interruption
+    enter-alien-callback
     #!+sb-thread sb!thread::enter-foreign-callback
     #!+(and sb-safepoint-strictly (not win32))
-    sb!unix::signal-handler-callback))
-
-(defparameter *common-static-symbols*
-  '(t
-
-    ;; filled in by the C code to propagate to Lisp
-    *posix-argv* *core-string*
+    sb!unix::signal-handler-callback)
+  #'equal)
 
+(defconstant-eqx +common-static-symbols+
+  `(t
     ;; free pointers.  Note that these are FIXNUM word counts, not (as
     ;; one might expect) byte counts or SAPs. The reason seems to be
     ;; that by representing them this way, we can avoid consing
@@ -187,14 +181,7 @@
     #!+(and sb-thread ppc) *tls-index-lock*
 
     ;; dynamic runtime linking support
-    #!+sb-dynamic-core *required-runtime-c-symbols*
-    sb!kernel::*gc-epoch*
-
-    ;; Dispatch tables for generic array access
-    sb!impl::%%data-vector-reffers%%
-    sb!impl::%%data-vector-reffers/check-bounds%%
-    sb!impl::%%data-vector-setters%%
-    sb!impl::%%data-vector-setters/check-bounds%%
+    #!+sb-dynamic-core +required-runtime-c-symbols+
 
     ;; non-x86oid gencgc object pinning
     #!+(and gencgc (not (or x86 x86-64)))
@@ -204,11 +191,19 @@
     ;; and patching them on runtime startup
     sb!fasl::*assembler-routines*
 
-    ;; hash table weaknesses
-    :key
-    :value
-    :key-and-value
-    :key-or-value))
+    ;;; The following symbols aren't strictly required to be static
+    ;;; - they are not accessed from C - but we make them static in order
+    ;;; to (perhaps) micro-optimize access in Lisp.
+    ;;; However there is no efficiency gain if we have #!+immobile-space.
+    #!-immobile-space ,@'(
+     ;; arbitrary object that changes after each GC
+     sb!kernel::*gc-epoch*
+     ;; Dispatch tables for generic array access
+     sb!impl::%%data-vector-reffers%%
+     sb!impl::%%data-vector-reffers/check-bounds%%
+     sb!impl::%%data-vector-setters%%
+     sb!impl::%%data-vector-setters/check-bounds%%))
+  #'equalp)
 
 ;;; Number of entries in the thread local storage. Limits the number
 ;;; of symbols with thread local bindings.
@@ -222,3 +217,8 @@
 (progn
   (defconstant +highest-normal-generation+ 5)
   (defconstant +pseudo-static-generation+ 6))
+
+(defun !unintern-symbols ()
+  '("SB-VM"
+    +c-callable-fdefns+
+    +common-static-symbols+))
