@@ -32,7 +32,7 @@
 
 (defun static-symbol-p (symbol)
   (or (null symbol)
-      (and (member symbol +static-symbols+) t)))
+      (and (find symbol +static-symbols+) t)))
 
 ;;; the byte offset of the static symbol SYMBOL
 (defun static-symbol-offset (symbol)
@@ -48,16 +48,22 @@
 ;;; Return the (byte) offset from NIL to the start of the fdefn object
 ;;; for the static function NAME.
 (defun static-fdefn-offset (name)
-  (let ((static-syms (length +static-symbols+))
-        (static-fun-index (position name +static-fdefns+)))
-    (unless static-fun-index
-      (error "~S isn't a static function." name))
-    (+ (* static-syms (pad-data-block symbol-size))
+  (let ((static-fun-index
+         (or (position name #.(concatenate 'vector +c-callable-fdefns+ +static-fdefns+))
+             (error "~S isn't a static function." name))))
+    (+ (* (length +static-symbols+) (pad-data-block symbol-size))
        (pad-data-block (1- symbol-size))
        (- list-pointer-lowtag)
        (* static-fun-index (pad-data-block fdefn-size))
        other-pointer-lowtag)))
 
+;;; Return absolute address of the 'fun' slot in static fdefn NAME.
+(defun static-fdefn-fun-addr (name)
+  (+ nil-value
+     (static-fdefn-offset name)
+     (- other-pointer-lowtag)
+     (ash fdefn-fun-slot word-shift)))
+
 ;;; Return the (byte) offset from NIL to the raw-addr slot of the
 ;;; fdefn object for the static function NAME.
 (defun static-fun-offset (name)
