@@ -75,12 +75,7 @@
     (flet ((zero-init-p (x)
              ;; dynamic-space is already zeroed
              (and (not dx-p)
-                  ;; KLUDGE: can't ignore type-derived
-                  ;; constants since they can be closed over
-                  ;; and not using them confuses the register
-                  ;; allocator.
-                  ;; See compiler.pure/cons-zero-initialization
-                  (strictly-constant-lvar-p x)
+                  (constant-lvar-p x)
                   (eql (lvar-value x) 0))))
      (dolist (init inits)
        (let ((kind (car init))
@@ -130,6 +125,12 @@
                         (aver args)
                         (lvar-tn node block (pop args)))
                        (:unbound
+                        ;; SLOT should be the word index to alter, but with structure
+                        ;; instances, SLOT is a cons whose car is the raw-slot-type
+                        ;; since BOXED-COMBINATION-REF-P expects that #'cddr is the
+                        ;; slot index.
+                        (when (listp slot)
+                          (setq slot (+ (cdr slot) sb!vm:instance-slots-offset)))
                         (or unbound-marker-tn
                             (setf unbound-marker-tn
                                   (let ((tn (make-restricted-tn
@@ -251,12 +252,7 @@
           (let ((value (pop initial-contents)))
             ;; dynamic-space is already zeroed
             (unless (and (not dx-p)
-                         ;; KLUDGE: can't ignore type-derived
-                         ;; constants since they can be closed over
-                         ;; and not using them confuses the register
-                         ;; allocator.
-                         ;; See compiler.pure/vector-zero-initialization
-                         (strictly-constant-lvar-p value)
+                         (constant-lvar-p value)
                          (if character
                              (eql (char-code (lvar-value value)) 0)
                              (eql (lvar-value value) 0)))
