@@ -20,57 +20,70 @@
 ;;;; function representation" before you seriously mess with this
 ;;;; stuff.
 
-;;; Verify that the NAME is a legal name for a variable and return a
-;;; VAR structure for it, filling in info if it is globally special.
-;;; If it is losing, we punt with a COMPILER-ERROR. NAMES-SO-FAR is a
-;;; list of names which have previously been bound. If the NAME is in
-;;; this list, then we error out.
-(declaim (ftype (sfunction (t list &optional t) lambda-var) varify-lambda-arg))
-(defun varify-lambda-arg (name names-so-far &optional (context "lambda list"))
-  (declare (inline member))
-  (unless (symbolp name) ;; FIXME: probably unreachable. Change to AVER?
-    (compiler-error "~S is not a symbol, and cannot be used as a variable." name))
-  (when (member name names-so-far :test #'eq)
-    (compiler-error "The variable ~S occurs more than once in the ~A."
-                    name
-                    context))
-  (let ((kind (info :variable :kind name)))
-    (cond ((keywordp name)
-           (compiler-error "~S is a keyword, and cannot be used as a local variable."
-                           name))
-          ((eq kind :constant)
-           (compiler-error "~@<~S names a defined constant, and cannot be used as a ~
-                            local variable.~:@>"
-                           name))
-          ((eq :global kind)
-           (compiler-error "~@<~S names a global lexical variable, and cannot be used ~
-                            as a local variable.~:@>"
-                           name))
-          ((eq kind :special)
-           (let ((specvar (find-free-var name)))
-             (make-lambda-var :%source-name name
-                              :type (leaf-type specvar)
-                              :where-from (leaf-where-from specvar)
-                              :specvar specvar)))
-          (t
-           (make-lambda-var :%source-name name)))))
+(declaim (ftype (sfunction * function) make-repeated-name-check))
+(defun make-repeated-name-check (&key
+                                   (kind "variable")
+                                   (context "lambda list")
+                                   (signal-via #'compiler-error))
+  (let ((seen '()))
+    (lambda (name)
+      (when (member name seen :test #'eq)
+        (funcall signal-via "~@<The ~A ~S occurs more than once in ~
+                             the ~A.~@:>"
+                 kind name context))
+      (push name seen)
+      name)))
 
-;;; Make the default keyword for a &KEY arg, checking that the keyword
-;;; isn't already used by one of the VARS.
-(declaim (ftype (sfunction (symbol list t) symbol) make-keyword-for-arg))
-(defun make-keyword-for-arg (symbol vars keywordify)
-  (let ((key (if (and keywordify (not (keywordp symbol)))
-                 (keywordicate symbol)
-                 symbol)))
-    (dolist (var vars)
-      (let ((info (lambda-var-arg-info var)))
-        (when (and info
-                   (eq (arg-info-kind info) :keyword)
-                   (eq (arg-info-key info) key))
-          (compiler-error
-           "The keyword ~S appears more than once in the lambda list."
-           key))))
-    key))
+;;; Verify that NAME is a legal name for a variable.
+(declaim (ftype (function (t &key
+                             (:context t) (:allow-special t) (:allow-symbol-macro t)
+                             (:signal-via (or symbol function)))
+                          (values symbol keyword))
+                check-variable-name-for-binding))
+(defun check-variable-name-for-binding (name
+                                        &key
+                                          context
+                                          (allow-special t)
+                                          (allow-symbol-macro t)
+                                          (signal-via #'compiler-error))
+  (check-variable-name name :signal-via signal-via)
+  (flet ((lose (kind)
+           (funcall signal-via
+                    #+xc-host "~@<~/sb!impl:print-symbol-with-prefix/ names a ~
+                               ~A, and cannot be used in ~A.~:@>"
+                    #-xc-host "~@<~/sb-impl:print-symbol-with-prefix/ names a ~
+                              ~A, and cannot be used in ~A.~:@>"
+                    name kind context)))
+    (let ((kind (info :variable :kind name)))
+      (case kind
+        (:macro
+         (unless allow-symbol-macro
+           (program-assert-symbol-home-package-unlocked
+            :compile name (format nil "lexically binding global ~
+                                       symbol-macro ~~A in ~A"
+                                  context))))
+        ((:constant)
+         (lose "defined constant"))
+        ((:global)
+         (lose "global lexical variable"))
+        (:special
+         (unless allow-special
+           (lose "special variable"))))
+      (values name kind))))
+
+;;; Return a VAR structure for NAME, filling in info if it is globally
+;;; special. If it is losing, we punt with a COMPILER-ERROR.
+(declaim (ftype (sfunction (symbol) lambda-var) varify-lambda-arg))
+(defun varify-lambda-arg (name)
+  (case (info :variable :kind name)
+    (:special
+     (let ((variable (find-free-var name)))
+       (make-lambda-var :%source-name name
+                        :type (leaf-type variable)
+                        :where-from (leaf-where-from variable)
+                        :specvar variable)))
+    (t
+     (make-lambda-var :%source-name name))))
 
 ;;; Parse a lambda list into a list of VAR structures, stripping off
 ;;; any &AUX bindings. Each arg name is checked for legality, and
@@ -88,94 +101,54 @@
                 make-lambda-vars))
 (defun make-lambda-vars (list)
   (multiple-value-bind (llks required optional rest/more keys aux)
-      (parse-lambda-list list)
+      (multiple-value-call #'check-lambda-list-names (parse-lambda-list list))
     (collect ((vars)
-              (names-so-far)
               (aux-vars)
               (aux-vals))
-      (flet (;; PARSE-DEFAULT deals with defaults and supplied-p args
-             ;; for optionals and keywords args.
-             (parse-default (spec info)
-               (when (consp (cdr spec))
-                 (setf (arg-info-default info) (second spec))
-                 (when (consp (cddr spec))
-                   (let* ((supplied-p (third spec))
-                          (supplied-var (varify-lambda-arg supplied-p
-                                                           (names-so-far))))
-                     (setf (arg-info-supplied-p info) supplied-var)
-                     (names-so-far supplied-p))))))
-
-        (dolist (name required)
-          (let ((var (varify-lambda-arg name (names-so-far))))
-            (vars var)
-            (names-so-far name)))
-
+      (flet ((add-var (name)
+               (let ((var (varify-lambda-arg name)))
+                 (vars var)
+                 var))
+             (add-info (var kind &key (default nil defaultp) suppliedp-var key)
+               (let ((info (make-arg-info :kind kind)))
+                 (when defaultp
+                   (setf (arg-info-default info) default))
+                 (when suppliedp-var
+                   (setf (arg-info-supplied-p info)
+                         (varify-lambda-arg suppliedp-var)))
+                 (when key
+                   (setf (arg-info-key info) key))
+                 (setf (lambda-var-arg-info var) info))))
+        ;; Required
+        (mapc #'add-var required)
+        ;; Optional
         (dolist (spec optional)
-          (if (atom spec)
-              (let ((var (varify-lambda-arg spec (names-so-far))))
-                (setf (lambda-var-arg-info var)
-                      (make-arg-info :kind :optional))
-                (vars var)
-                (names-so-far spec))
-              (let* ((name (first spec))
-                     (var (varify-lambda-arg name (names-so-far)))
-                     (info (make-arg-info :kind :optional)))
-                (setf (lambda-var-arg-info var) info)
-                (vars var)
-                (names-so-far name)
-                (parse-default spec info))))
-
+          (multiple-value-bind (name default suppliedp-var defaultp)
+              (parse-optional-arg-spec spec)
+            (apply #'add-info (add-var name) :optional
+                   :suppliedp-var (first suppliedp-var)
+                   (when defaultp (list :default default)))))
+        ;; Rest/more
         (when rest/more
           (mapc (lambda (name kind)
-                  (let ((var (varify-lambda-arg name (names-so-far))))
-                    (setf (lambda-var-arg-info var) (make-arg-info :kind kind))
-                    (vars var)
-                    (names-so-far name)))
+                  (add-info (add-var name) kind))
                 rest/more (let ((morep (eq (ll-kwds-restp llks) '&more)))
                             (if morep '(:more-context :more-count) '(:rest)))))
-
+        ;; Keys
         (dolist (spec keys)
-          (cond
-           ((atom spec)
-            (let ((var (varify-lambda-arg spec (names-so-far))))
-              (setf (lambda-var-arg-info var)
-                    (make-arg-info :kind :keyword
-                                   :key (make-keyword-for-arg spec
-                                                              (vars)
-                                                              t)))
-              (vars var)
-              (names-so-far spec)))
-           ((atom (first spec))
-            (let* ((name (first spec))
-                   (var (varify-lambda-arg name (names-so-far)))
-                   (info (make-arg-info
-                          :kind :keyword
-                          :key (make-keyword-for-arg name (vars) t))))
-              (setf (lambda-var-arg-info var) info)
-              (vars var)
-              (names-so-far name)
-              (parse-default spec info)))
-           (t
-            (let ((head (first spec)))
-              (let* ((name (second head))
-                     (var (varify-lambda-arg name (names-so-far)))
-                     (info (make-arg-info
-                            :kind :keyword
-                            :key (make-keyword-for-arg (first head)
-                                                       (vars)
-                                                       nil))))
-                (setf (lambda-var-arg-info var) info)
-                (vars var)
-                (names-so-far name)
-                (parse-default spec info))))))
-
+          (multiple-value-bind (keyword name default suppliedp-var defaultp)
+              (parse-key-arg-spec spec)
+            (apply #'add-info (add-var name) :keyword
+                   :suppliedp-var (first suppliedp-var)
+                   :key keyword
+                   (when defaultp (list :default default)))))
+        ;; Aux
         (dolist (spec aux)
           (multiple-value-bind (name val)
               (if (atom spec) spec (values (car spec) (cadr spec)))
-            (let ((var (varify-lambda-arg name nil)))
+            (let ((var (varify-lambda-arg name)))
               (aux-vars var)
-              (aux-vals val)
-              (names-so-far name))))
+              (aux-vals val))))
 
         (values (vars) (ll-kwds-keyp llks) (ll-kwds-allowp llks)
                 (aux-vars) (aux-vals))))))
@@ -468,7 +441,12 @@
                                   default-vars default-vals
                                   (if supplied-p (list default nil) (list default))
                                   name)
-          (let* ((default `',(constant-form-value default))
+          (let* ((value (constant-form-value default))
+                 ;; One-and-only-once-more: MAYBE-EMIT-MAKE-LOAD-FORMS has a similar test.
+                 (namedp-not-eql-comparable
+                  (and (symbolp default)
+                       (not (typep value '(or symbol character number)))))
+                 (default (if namedp-not-eql-comparable default `',value))
                  (defaults (if supplied-p (list default nil) (list default))))
             ;; DEFAULT can contain a reference to a
             ;; to-be-optimized-away function/block/tag, so better to
@@ -519,10 +497,12 @@
                                  :where-from (leaf-where-from var))))
 
     (let* ((n-context (sb!xc:gensym "N-CONTEXT-"))
-           (context-temp (make-lambda-var :%source-name n-context))
+           (context-temp (make-lambda-var :%source-name n-context
+                                          :arg-info (make-arg-info :kind :more-context)))
            (n-count (sb!xc:gensym "N-COUNT-"))
            (count-temp (make-lambda-var :%source-name n-count
-                                        :type (specifier-type 'index))))
+                                        :type (specifier-type 'index)
+                                        :arg-info (make-arg-info :kind :more-count))))
 
       (arg-vars context-temp count-temp)
 
