@@ -120,6 +120,22 @@
 
 ;;;; generic type inference methods
 
+(defun symeval-derive-type (node &aux (args (basic-combination-args node))
+                                      (lvar (pop args)))
+  (unless (and lvar (endp args))
+    (return-from symeval-derive-type))
+  (if (constant-lvar-p lvar)
+      (let* ((sym (lvar-value lvar))
+             (var (maybe-find-free-var sym))
+             (local-type (when var
+                           (let ((*lexenv* (node-lexenv node)))
+                             (lexenv-find var type-restrictions))))
+             (global-type (info :variable :type sym)))
+        (if local-type
+            (type-intersection local-type global-type)
+            global-type))
+      *universal-type*))
+
 ;;; Derive the type to be the type of the xxx'th arg. This can normally
 ;;; only be done when the result value is that argument.
 (defun result-type-first-arg (call)
