@@ -707,6 +707,20 @@
       ,node
       #'closure-needing-ir1-environment-from-node)))
 
+;;; *SOURCE-PATHS* is a hashtable from source code forms to the path
+;;; taken through the source to reach the form. This provides a way to
+;;; keep track of the location of original source forms, even when
+;;; macroexpansions and other arbitary permutations of the code
+;;; happen. This table is initialized by calling FIND-SOURCE-PATHS on
+;;; the original source.
+;;;
+;;; It is fairly useless to store symbols, characters, or fixnums in
+;;; this table, as 42 is EQ to 42 no matter where in the source it
+;;; appears. GET-SOURCE-PATH and NOTE-SOURCE-PATH functions should be
+;;; always used to access this table.
+(declaim (hash-table *source-paths*))
+(defvar *source-paths*)
+
 (defmacro with-source-paths (&body forms)
   (with-unique-names (source-paths)
     `(let* ((,source-paths (make-hash-table :test 'eq))
