@@ -13,18 +13,6 @@
 
 (in-package "SB!C")
 
-;;; FIXME: Doesn't this belong somewhere else, like early-c.lisp?
-(declaim (special *constants* *free-vars* *component-being-compiled*
-                  *free-funs* *source-paths*
-                  *undefined-warnings* *compiler-error-count*
-                  *compiler-warning-count* *compiler-style-warning-count*
-                  *compiler-note-count*
-                  *compiler-error-bailout*
-                  *last-format-string* *last-format-args*
-                  *last-message-count* *last-error-context*
-                  *lexenv* *fun-names-in-this-file*
-                  *allow-instrumenting*))
-
 ;;; Whether reference to a thing which cannot be defined causes a full
 ;;; warning.
 (defvar *flame-on-necessarily-undefined-thing* nil)
@@ -98,9 +86,8 @@
     (pprint-logical-block (*standard-output* nil :per-line-prefix "; ")
        (apply #'compiler-mumble foo))))
 
-(deftype object () '(or fasl-output core-object null))
 
-(defvar *compile-object* nil)
+(deftype object () '(or fasl-output core-object null))
 (declaim (type object *compile-object*))
 (defvar *compile-toplevel-object* nil)
 
@@ -1288,6 +1275,8 @@ necessary, since type inference may take arbitrarily long to converge.")
               ;; EVAL strategy of compiling everything inside (LAMBDA ()
               ;; ...).  -- CSR, 2002-11-02
               (when (core-object-p *compile-object*)
+                #+sb-xc-host (error "Can't compile to core")
+                #-sb-xc-host
                 (fix-core-source-info *source-info* *compile-object*
                                       (and (policy (lambda-bind fun)
                                                (> eval-store-source-form 0))
@@ -1697,6 +1686,8 @@ necessary, since type inference may take arbitrarily long to converge.")
                          (lexenv-handled-conditions *lexenv*))))
       (and ctype (handle-p condition (car ctype))))))
 
+(defvar *fun-names-in-this-file* nil)
+
 ;;; Read all forms from INFO and compile them, with output to
 ;;; *COMPILE-OBJECT*. Return (VALUES ABORT-P WARNINGS-P FAILURE-P).
 (defun sub-compile-file (info)
