@@ -435,7 +435,8 @@ and a pointer to the arguments."
             (inst sw ra sp (- n-frame-bytes n-word-bytes))
 
             ;; Setup the args and make the call.
-            (inst li a0 (get-lisp-obj-address #'enter-alien-callback))
+            (inst li a0 (static-fdefn-fun-addr 'enter-alien-callback))
+            (inst lw a0 a0)
             (inst li t9 (foreign-symbol-address "funcall3"))
             (inst li a1 (fixnumize index))
             (inst addu a2 sp n-frame-bytes)
