@@ -55,7 +55,7 @@
 ;;; Symbol-Value of NIL is NIL.
 ;;;
 (define-vop (symbol-value checked-cell-ref)
-  (:translate symbol-value)
+  (:translate symeval)
   (:generator 9
     (move obj-temp object)
     (loadw value obj-temp symbol-value-slot other-pointer-lowtag)
@@ -86,7 +86,7 @@
 (define-vop (fast-symbol-value cell-ref)
   (:variant symbol-value-slot other-pointer-lowtag)
   (:policy :fast)
-  (:translate symbol-value))
+  (:translate symeval))
 
 (define-vop (symbol-hash)
   (:policy :fast-safe)
@@ -107,9 +107,9 @@
 ;;; On unithreaded builds these are just copies of the non-global versions.
 (define-vop (%set-symbol-global-value set))
 (define-vop (symbol-global-value symbol-value)
-  (:translate symbol-global-value))
+  (:translate sym-global-val))
 (define-vop (fast-symbol-global-value fast-symbol-value)
-  (:translate symbol-global-value))
+  (:translate sym-global-val))
 
 ;;;; Fdefinition (fdefn) objects.
 
@@ -143,12 +143,12 @@
     (let ((normal-fn (gen-label)))
       (load-type type function (- fun-pointer-lowtag))
       (inst nop)
-      (inst xor type simple-fun-header-widetag)
+      (inst xor type simple-fun-widetag)
       (inst beq type normal-fn)
       (inst addu lip function
             (- (ash simple-fun-code-offset word-shift)
                fun-pointer-lowtag))
-      (inst li lip (make-fixup "closure_tramp" :foreign))
+      (inst li lip (make-fixup 'closure-tramp :assembly-routine))
       (emit-label normal-fn)
       (storew lip fdefn fdefn-raw-addr-slot other-pointer-lowtag)
       (storew function fdefn fdefn-fun-slot other-pointer-lowtag)
@@ -162,7 +162,7 @@
   (:results (result :scs (descriptor-reg)))
   (:generator 38
     (storew null-tn fdefn fdefn-fun-slot other-pointer-lowtag)
-    (inst li temp (make-fixup "undefined_tramp" :foreign))
+    (inst li temp (make-fixup 'undefined-tramp :assembly-routine))
     (storew temp fdefn fdefn-raw-addr-slot other-pointer-lowtag)
     (move result fdefn)))
 
@@ -175,7 +175,7 @@
 ;;; symbol.
 ;;; See the "Chapter 9: Specials" of the SBCL Internals Manual.
 
-(define-vop (bind)
+(define-vop (dynbind)
   (:args (val :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg)))
   (:temporary (:scs (descriptor-reg)) temp)
@@ -303,18 +303,11 @@
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
   (:results (value :scs (unsigned-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types unsigned-num)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset n-word-bytes)
-    (inst addu lip offset object)
-    (inst lw value lip (- (* instance-slots-offset n-word-bytes)
-                          instance-pointer-lowtag))))
+    (inst addu lip object index)
+    (loadw value lip instance-slots-offset instance-pointer-lowtag)))
 
 (define-vop (raw-instance-set/word)
   (:translate %raw-instance-set/word)
@@ -324,18 +317,11 @@
          (value :scs (unsigned-reg) :target result))
   (:arg-types * positive-fixnum unsigned-num)
   (:results (result :scs (unsigned-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types unsigned-num)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset n-word-bytes)
-    (inst addu lip offset object)
-    (inst sw value lip (- (* instance-slots-offset n-word-bytes)
-                          instance-pointer-lowtag))
+    (inst addu lip object index)
+    (storew value lip instance-slots-offset instance-pointer-lowtag)
     (move result value)))
 
 (define-vop (raw-instance-ref/single)
@@ -345,16 +331,10 @@
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
   (:results (value :scs (single-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types single-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset n-word-bytes)
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (inst lwc1 value lip (- (* instance-slots-offset n-word-bytes)
                             instance-pointer-lowtag))))
 
@@ -366,16 +346,10 @@
          (value :scs (single-reg) :target result))
   (:arg-types * positive-fixnum single-float)
   (:results (result :scs (single-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types single-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset n-word-bytes)
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (inst swc1 value lip (- (* instance-slots-offset n-word-bytes)
                             instance-pointer-lowtag))
     (unless (location= result value)
@@ -388,26 +362,20 @@
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
   (:results (value :scs (double-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types double-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset (* 2 n-word-bytes))
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (let ((immediate-offset (- (* instance-slots-offset n-word-bytes)
                                instance-pointer-lowtag)))
       (ecase *backend-byte-order*
-        (:big-endian (inst lwc1 value lip immediate-offset))
-        (:little-endian (inst lwc1-odd value lip immediate-offset))))
+        (:big-endian (inst lwc1-odd value lip immediate-offset))
+        (:little-endian (inst lwc1 value lip immediate-offset))))
     (let ((immediate-offset (- (* (1+ instance-slots-offset) n-word-bytes)
                                instance-pointer-lowtag)))
       (ecase *backend-byte-order*
-        (:big-endian (inst lwc1-odd value lip immediate-offset))
-        (:little-endian (inst lwc1 value lip immediate-offset))))))
+        (:big-endian (inst lwc1 value lip immediate-offset))
+        (:little-endian (inst lwc1-odd value lip immediate-offset))))))
 
 (define-vop (raw-instance-set/double)
   (:translate %raw-instance-set/double)
@@ -417,26 +385,20 @@
          (value :scs (double-reg) :target result))
   (:arg-types * positive-fixnum double-float)
   (:results (result :scs (double-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types double-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset (* 2 n-word-bytes))
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (let ((immediate-offset (- (* instance-slots-offset n-word-bytes)
                                instance-pointer-lowtag)))
-      (ecase *backend-byte-order*
-        (:big-endian (inst swc1 value lip immediate-offset))
-        (:little-endian (inst swc1-odd value lip immediate-offset))))
-    (let ((immediate-offset (- (* (1+ instance-slots-offset) n-word-bytes)
-                               instance-pointer-lowtag)))
       (ecase *backend-byte-order*
         (:big-endian (inst swc1-odd value lip immediate-offset))
         (:little-endian (inst swc1 value lip immediate-offset))))
+    (let ((immediate-offset (- (* (1+ instance-slots-offset) n-word-bytes)
+                               instance-pointer-lowtag)))
+      (ecase *backend-byte-order*
+        (:big-endian (inst swc1 value lip immediate-offset))
+        (:little-endian (inst swc1-odd value lip immediate-offset))))
     (unless (location= result value)
       (inst fmove :double result value))))
 
@@ -447,16 +409,10 @@
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
   (:results (value :scs (complex-single-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types complex-single-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset (* 2 n-word-bytes))
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (inst lwc1
           (complex-single-reg-real-tn value)
           lip
@@ -476,16 +432,10 @@
          (value :scs (complex-single-reg) :target result))
   (:arg-types * positive-fixnum complex-single-float)
   (:results (result :scs (complex-single-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types complex-single-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset (* 2 n-word-bytes))
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (let ((value-real (complex-single-reg-real-tn value))
           (result-real (complex-single-reg-real-tn result)))
       (inst swc1
@@ -512,57 +462,51 @@
          (index :scs (any-reg)))
   (:arg-types * positive-fixnum)
   (:results (value :scs (complex-double-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types complex-double-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset (* 4 n-word-bytes))
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (let ((immediate-offset (- (* instance-slots-offset n-word-bytes)
                                instance-pointer-lowtag)))
       (ecase *backend-byte-order*
-        (:big-endian (inst lwc1
+        (:big-endian (inst lwc1-odd
                            (complex-double-reg-real-tn value)
                            lip
                            immediate-offset))
-        (:little-endian (inst lwc1-odd
+        (:little-endian (inst lwc1
                               (complex-double-reg-real-tn value)
                               lip
                               immediate-offset))))
     (let ((immediate-offset (- (* (1+ instance-slots-offset) n-word-bytes)
                                instance-pointer-lowtag)))
       (ecase *backend-byte-order*
-        (:big-endian (inst lwc1-odd
+        (:big-endian (inst lwc1
                            (complex-double-reg-real-tn value)
                            lip
                            immediate-offset))
-        (:little-endian (inst lwc1
+        (:little-endian (inst lwc1-odd
                               (complex-double-reg-real-tn value)
                               lip
                               immediate-offset))))
     (let ((immediate-offset (- (* (+ instance-slots-offset 2) n-word-bytes)
                                instance-pointer-lowtag)))
+      (ecase *backend-byte-order*
+        (:big-endian (inst lwc1-odd
+                           (complex-double-reg-imag-tn value)
+                           lip
+                           immediate-offset))
+        (:little-endian (inst lwc1
+                              (complex-double-reg-imag-tn value)
+                              lip
+                              immediate-offset))))
+    (let ((immediate-offset (- (* (+ instance-slots-offset 3) n-word-bytes)
+                               instance-pointer-lowtag)))
       (ecase *backend-byte-order*
         (:big-endian (inst lwc1
                            (complex-double-reg-imag-tn value)
                            lip
                            immediate-offset))
         (:little-endian (inst lwc1-odd
-                              (complex-double-reg-imag-tn value)
-                              lip
-                              immediate-offset))))
-    (let ((immediate-offset (- (* (+ instance-slots-offset 3) n-word-bytes)
-                               instance-pointer-lowtag)))
-      (ecase *backend-byte-order*
-        (:big-endian (inst lwc1-odd
-                           (complex-double-reg-imag-tn value)
-                           lip
-                           immediate-offset))
-        (:little-endian (inst lwc1
                               (complex-double-reg-imag-tn value)
                               lip
                               immediate-offset))))))
@@ -575,31 +519,14 @@
          (value :scs (complex-double-reg) :target result))
   (:arg-types * positive-fixnum complex-double-float)
   (:results (result :scs (complex-double-reg)))
-  (:temporary (:scs (non-descriptor-reg)) offset)
   (:temporary (:scs (interior-reg)) lip)
   (:result-types complex-double-float)
   (:generator 5
-    (loadw offset object 0 instance-pointer-lowtag)
-    (inst srl offset n-widetag-bits)
-    (inst sll offset n-fixnum-tag-bits)
-    (inst subu offset index)
-    (inst subu offset (* 4 n-word-bytes))
-    (inst addu lip offset object)
+    (inst addu lip object index)
     (let ((value-real (complex-double-reg-real-tn value))
           (result-real (complex-double-reg-real-tn result)))
       (let ((immediate-offset (- (* instance-slots-offset n-word-bytes)
                                  instance-pointer-lowtag)))
-        (ecase *backend-byte-order*
-          (:big-endian (inst swc1
-                             value-real
-                             lip
-                             immediate-offset))
-          (:little-endian (inst swc1-odd
-                                value-real
-                                lip
-                                immediate-offset))))
-      (let ((immediate-offset (- (* (1+ instance-slots-offset) n-word-bytes)
-                                 instance-pointer-lowtag)))
         (ecase *backend-byte-order*
           (:big-endian (inst swc1-odd
                              value-real
@@ -609,23 +536,23 @@
                                 value-real
                                 lip
                                 immediate-offset))))
+      (let ((immediate-offset (- (* (1+ instance-slots-offset) n-word-bytes)
+                                 instance-pointer-lowtag)))
+        (ecase *backend-byte-order*
+          (:big-endian (inst swc1
+                             value-real
+                             lip
+                             immediate-offset))
+          (:little-endian (inst swc1-odd
+                                value-real
+                                lip
+                                immediate-offset))))
       (unless (location= result-real value-real)
         (inst fmove :double result-real value-real)))
     (let ((value-imag (complex-double-reg-imag-tn value))
           (result-imag (complex-double-reg-imag-tn result)))
       (let ((immediate-offset (- (* (+ instance-slots-offset 2) n-word-bytes)
                                  instance-pointer-lowtag)))
-        (ecase *backend-byte-order*
-          (:big-endian (inst swc1
-                             value-imag
-                             lip
-                             immediate-offset))
-          (:little-endian (inst swc1-odd
-                                value-imag
-                                lip
-                                immediate-offset))))
-      (let ((immediate-offset (- (* (+ instance-slots-offset 3) n-word-bytes)
-                                 instance-pointer-lowtag)))
         (ecase *backend-byte-order*
           (:big-endian (inst swc1-odd
                              value-imag
@@ -635,5 +562,16 @@
                                 value-imag
                                 lip
                                 immediate-offset))))
+      (let ((immediate-offset (- (* (+ instance-slots-offset 3) n-word-bytes)
+                                 instance-pointer-lowtag)))
+        (ecase *backend-byte-order*
+          (:big-endian (inst swc1
+                             value-imag
+                             lip
+                             immediate-offset))
+          (:little-endian (inst swc1-odd
+                                value-imag
+                                lip
+                                immediate-offset))))
       (unless (location= result-imag value-imag)
         (inst fmove :double result-imag value-imag)))))
