@@ -122,7 +122,8 @@
   (defconstant static-space-end          #x111ff000)
 
   (defconstant dynamic-space-start       #x30000000)
-  (defconstant dynamic-space-end         (!configure-dynamic-space-end)))
+  (defconstant default-dynamic-space-size
+    (or #.(!read-dynamic-space-size) (expt 2 29))))
 
 #!+(and sunos cheneygc) ; might as well start by trying the same numbers
 (progn
@@ -153,7 +154,8 @@
   (defconstant static-space-end          #x111ff000)
 
   (defconstant dynamic-space-start       #x30000000)
-  (defconstant dynamic-space-end         (!configure-dynamic-space-end)))
+  (defconstant default-dynamic-space-size
+    (or #.(!read-dynamic-space-size) (expt 2 29))))
 
 #!+netbsd ; Need a gap at 0x4000000 for shared libraries
 (progn
@@ -201,18 +203,18 @@
 ;;; space directly after the static symbols.  That way, the raw-addr
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 ;;;
-(defparameter *static-symbols*
-  (append
-   *common-static-symbols*
-   *c-callable-static-symbols*
-   '(#!+gencgc *restart-lisp-function*)))
+(defconstant-eqx +static-symbols+
+  `#(,@+common-static-symbols+
+     #!+gencgc *restart-lisp-function*)
+  #'equalp)
 
-(defparameter *static-funs*
-  '(length
+(defconstant-eqx +static-fdefns+
+  #(length
     two-arg-+ two-arg-- two-arg-* two-arg-/ two-arg-< two-arg-> two-arg-=
     two-arg-<= two-arg->= two-arg-/= eql %negate
     two-arg-and two-arg-ior two-arg-xor two-arg-eqv
-    two-arg-gcd two-arg-lcm))
+    two-arg-gcd two-arg-lcm)
+  #'equalp)
 
 ;;;; Pseudo-atomic trap number
 
