@@ -1231,8 +1231,7 @@
   (declare (type sb!di:debug-fun debug-fun))
   (let ((sc-vec sb!c::*backend-sc-numbers*)
         (groups nil)
-        (debug-vars (sb!di::debug-fun-debug-vars
-                     debug-fun)))
+        (debug-vars (sb!di::debug-fun-debug-vars debug-fun)))
     (and debug-vars
          (dotimes (debug-var-offset
                    (length debug-vars)
@@ -1804,7 +1803,7 @@
     (setf *assembler-routines-by-addr*
           (invert-address-hash *static-foreign-symbols*
                                *assembler-routines-by-addr*))
-    (loop for name in sb!vm:*static-funs*
+    (loop for name across sb!vm:+static-fdefns+
           for address =
           #!+immobile-code (sb!vm::function-raw-address name)
           #!-immobile-code (+ sb!vm::nil-value (sb!vm::static-fun-offset name))
@@ -1860,7 +1859,7 @@
 (defun note (note dstate)
   (declare (type (or string function) note)
            (type disassem-state dstate))
-  (push note (dstate-notes dstate)))
+  (setf (dstate-notes dstate) (nconc (dstate-notes dstate) (list note))))
 
 (defun prin1-short (thing stream)
   (with-print-restrictions
@@ -1997,7 +1996,7 @@
       t)))
 
 (defun maybe-note-static-symbol (address dstate)
-  (dolist (symbol sb!vm:*static-symbols*)
+  (dovector (symbol sb!vm:+static-symbols+)
     (when (= (get-lisp-obj-address symbol) address)
       (return (note (lambda (s) (prin1 symbol s)) dstate))))
   ;; Guess whether 'address' is an immobile-space symbol by looking at
