@@ -447,21 +447,15 @@
         #!-sb-thread
         (progn
           ;; arg0 to FUNCALL3 (function)
-          ;;
-          ;; Indirect the access to ENTER-ALIEN-CALLBACK through
-          ;; the symbol-value slot of SB-ALIEN::*ENTER-ALIEN-CALLBACK*
-          ;; to ensure it'll work even if the GC moves ENTER-ALIEN-CALLBACK.
-          ;; Skip any SB-THREAD TLS magic, since we don't expect anyone
-          ;; to rebind the variable. -- JES, 2006-01-01
-          (inst mov rdi (+ nil-value (static-symbol-offset
-                                      'sb!alien::*enter-alien-callback*)))
-          (loadw rdi rdi symbol-value-slot other-pointer-lowtag)
+          (inst mov rdi (make-ea :qword :disp (static-fdefn-fun-addr 'enter-alien-callback)))
           ;; arg0 to ENTER-ALIEN-CALLBACK (trampoline index)
           (inst mov rsi (fixnumize index))
           ;; arg1 to ENTER-ALIEN-CALLBACK (pointer to argument vector)
           (inst mov rdx rsp)
           ;; add room on stack for return value
-          (inst sub rsp 8)
+          (inst sub rsp (if (evenp arg-count)
+                            (* n-word-bytes 2)
+                            n-word-bytes))
           ;; arg2 to ENTER-ALIEN-CALLBACK (pointer to return value)
           (inst mov rcx rsp)
 
@@ -484,7 +478,9 @@
           ;; arg1 to ENTER-ALIEN-CALLBACK (pointer to argument vector)
           (inst mov #!-win32 rsi #!+win32 rdx rsp)
           ;; add room on stack for return value
-          (inst sub rsp 8)
+          (inst sub rsp (if (evenp arg-count)
+                            (* n-word-bytes 2)
+                            n-word-bytes))
           ;; arg2 to ENTER-ALIEN-CALLBACK (pointer to return value)
           (inst mov #!-win32 rdx #!+win32 r8 rsp)
           ;; Make new frame
@@ -515,7 +511,13 @@
 
         ;; Pop the arguments and the return value from the stack to get
         ;; the return address at top of stack.
-        (inst add rsp (* (1+ (length argument-types)) n-word-bytes))
+
+        (inst add rsp (* (+ arg-count
+                            ;; Plus the return value and make sure it's aligned
+                            (if (evenp arg-count)
+                                2
+                                1))
+                         n-word-bytes))
         ;; Return
         (inst ret))
       (finalize-segment segment)
