@@ -100,7 +100,6 @@
          (error "~S is neither a type nor a specializer." specl))))
 
 (defun type-class (type)
-  (declare (special *the-class-t*))
   (setq type (type-from-specializer type))
   (if (atom type)
       (if (eq type t)
@@ -180,8 +179,7 @@
                (subtypep (convert-to-system-type type1)
                          (convert-to-system-type type2))))))))
 
-(defvar *built-in-class-symbols* ())
-(defvar *built-in-wrapper-symbols* ())
+(define-load-time-global *built-in-class-symbols* ())
 
 (defun get-built-in-class-symbol (class-name)
   (or (cadr (assq class-name *built-in-class-symbols*))
@@ -207,7 +205,7 @@
 ;;; Grovel over SB-KERNEL::*!BUILT-IN-CLASSES* in order to set
 ;;; SB-PCL:*BUILT-IN-CLASSES*.
 (/show "about to set up SB-PCL::*BUILT-IN-CLASSES*")
-(defvar *built-in-classes*
+(define-load-time-global *built-in-classes*
   (labels ((direct-supers (class)
              (/noshow "entering DIRECT-SUPERS" (classoid-name class))
              (if (typep class 'built-in-classoid)
@@ -294,10 +292,6 @@
 (defclass structure-object (slot-object) ()
   (:metaclass structure-class))
 
-(defstruct (dead-beef-structure-object
-            (:constructor |STRUCTURE-OBJECT class constructor|)
-            (:copier nil)))
-
 (defclass standard-object (slot-object) ())
 
 (defclass funcallable-standard-object (function standard-object)
