@@ -23,6 +23,7 @@
 #include "interr.h"
 #include "breakpoint.h"
 #include "monitor.h"
+#include "pseudo-atomic.h"
 
 void arch_init(void)
 {
@@ -36,19 +37,22 @@ os_vm_address_t arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *contex
 
 void arch_skip_instruction(os_context_t *context)
 {
-    /* KLUDGE: Other platforms check for trap codes and skip inlined
-     * trap/error parameters.  We should too. */
-
-    /* Note that we're doing integer arithmetic here, not pointer. So
-     * the value that the return value of os_context_pc_addr() points
-     * to will be incremented by 4, not 32.
-     */
+    u32 trap_instruction = *((u32 *)*os_context_pc_addr(context));
+    unsigned code = trap_instruction >> 5 & 0xFF;
     *os_context_pc_addr(context) += 4;
+    switch (code)
+    {
+    case trap_Error:
+    case trap_Cerror:
+        skip_internal_error(context);
+    default:
+        break;
+    }
 }
 
 unsigned char *arch_internal_error_arguments(os_context_t *context)
 {
-    return (unsigned char *)(*os_context_pc_addr(context) + 5);
+    return (unsigned char *)*os_context_pc_addr(context);
 }
 
 boolean arch_pseudo_atomic_atomic(os_context_t *context)
@@ -64,7 +68,7 @@ boolean arch_pseudo_atomic_atomic(os_context_t *context)
      * to arch_pseudo_atomic_atomic, but this seems clearer.
      * --NS 2007-05-15 */
 #ifdef LISP_FEATURE_GENCGC
-    return SymbolValue(PSEUDO_ATOMIC_ATOMIC, 0) != NIL;
+    return get_pseudo_atomic_atomic(arch_os_get_current_thread());
 #else
     return (!foreign_function_call_active)
         && (NIL != SymbolValue(PSEUDO_ATOMIC_ATOMIC,0));
@@ -73,12 +77,12 @@ boolean arch_pseudo_atomic_atomic(os_context_t *context)
 
 void arch_set_pseudo_atomic_interrupted(os_context_t *context)
 {
-    SetSymbolValue(PSEUDO_ATOMIC_INTERRUPTED, do_pending_interrupt, 0);
+    set_pseudo_atomic_interrupted(arch_os_get_current_thread());
 }
 
 void arch_clear_pseudo_atomic_interrupted(os_context_t *context)
 {
-    SetSymbolValue(PSEUDO_ATOMIC_INTERRUPTED, 0, 0);
+    clear_pseudo_atomic_interrupted(arch_os_get_current_thread());
 }
 
 unsigned int arch_install_breakpoint(void *pc)
@@ -113,13 +117,7 @@ arch_handle_fun_end_breakpoint(os_context_t *context)
 void
 arch_handle_single_step_trap(os_context_t *context, int trap)
 {
-    unsigned char register_offset =
-        *((unsigned char *)(*os_context_pc_addr(context))+5);
-    handle_single_step_trap(context, trap, register_offset);
-    /* KLUDGE: arch_skip_instruction() only skips one instruction, and
-     * there is a following word to deal with as well, so skip
-     * twice. */
-    arch_skip_instruction(context);
+    handle_single_step_trap(context, trap, reg_LEXENV);
     arch_skip_instruction(context);
 }
 
@@ -151,7 +149,7 @@ void arch_install_interrupt_handlers()
 
 #define LINKAGE_TEMP_REG        reg_NFP
 
-void arch_write_linkage_table_jmp(void* reloc_addr, void *target_addr)
+void arch_write_linkage_table_jmp(char *reloc_addr, void *target_addr)
 {
   /*
     ldr reg,=address
@@ -174,7 +172,7 @@ void arch_write_linkage_table_jmp(void* reloc_addr, void *target_addr)
   // address
   *(unsigned long *)inst_ptr++ = target_addr;
 
-  os_flush_icache((os_vm_address_t) reloc_addr, (char*) inst_ptr - (char*) reloc_addr);
+  os_flush_icache((os_vm_address_t) reloc_addr, (char*) inst_ptr - reloc_addr);
 }
 
 void
