@@ -185,6 +185,23 @@ sword_t scavenge(lispobj *start, sword_t n_words)
     return n_words;
 }
 
+void scav_binding_stack(lispobj* where, lispobj* end)
+{
+#ifdef LISP_FEATURE_SB_THREAD
+    // The binding stack stores TLS indices where symbols would be,
+    // and there's no reason to scavenge those words since they're fixnums.
+    // This means a symbol can not be enlivened if it exists *solely* on
+    // the binding stack - which is, practically speaking, impossible.
+    lispobj *object_ptr;
+    for (object_ptr = where; object_ptr < end; object_ptr += 2) {
+        lispobj object = *object_ptr;
+        if (is_lisp_pointer(object)) scav1(object_ptr, object);
+    }
+#else
+    scavenge(where, end - where);
+#endif
+}
+
 static lispobj trans_fun_header(lispobj object); /* forward decls */
 static lispobj trans_short_boxed(lispobj object);
 
@@ -1289,14 +1306,14 @@ gc_search_space3(void *pointer, lispobj *start, void *limit)
         else
             forwarded_start = start;
         lispobj thing = *forwarded_start;
-        count = is_cons_half(thing) ? 2 : sizetab[widetag_of(thing)](forwarded_start);
+        count = OBJECT_SIZE(thing, forwarded_start);
         /* Check whether the pointer is within this object. */
         if (pointer < (void*)(start+count)) return start;
     }
 #else
     for ( ; (void*)start < limit ; start += count) {
         lispobj thing = *start;
-        count = is_cons_half(thing) ? 2 : sizetab[widetag_of(thing)](start);
+        count = OBJECT_SIZE(thing, start);
         /* Check whether the pointer is within this object. */
         if (pointer < (void*)(start+count)) return start;
     }
