@@ -37,6 +37,7 @@
  * instead. See hpux-os.c for some useful restrictions on actual
  * usage. */
 
+#ifdef LISP_FEATURE_CHENEYGC
 void
 os_zero(os_vm_address_t addr, os_vm_size_t length)
 {
@@ -70,6 +71,7 @@ os_zero(os_vm_address_t addr, os_vm_size_t length)
                  addr);
     }
 }
+#endif
 
 os_vm_address_t
 os_allocate(os_vm_size_t len)
@@ -147,27 +149,6 @@ void* os_dlopen(char* name, int flags) {
  * table entry for each element of REQUIRED_RUNTIME_C_SYMBOLS.
  */
 
-/* We start with a little greenspunning to make car, cdr and base-string data
- * accessible. */
-
-/* Object tagged? (dereference (cast (untag (obj)))) */
-#define FOLLOW(obj,lowtagtype,ctype)            \
-    (*(struct ctype*)(obj - lowtagtype##_LOWTAG))
-
-/* For all types sharing OTHER_POINTER_LOWTAG: */
-#define FOTHERPTR(obj,ctype)                    \
-    FOLLOW(obj,OTHER_POINTER,ctype)
-
-static inline lispobj car(lispobj conscell)
-{
-    return FOLLOW(conscell,LIST_POINTER,cons).car;
-}
-
-static inline lispobj cdr(lispobj conscell)
-{
-    return FOLLOW(conscell,LIST_POINTER,cons).cdr;
-}
-
 #ifndef LISP_FEATURE_WIN32
 void *
 os_dlsym_default(char *name)
@@ -180,24 +161,23 @@ os_dlsym_default(char *name)
 
 void os_link_runtime()
 {
-    lispobj head;
     void *link_target = (void*)(intptr_t)LINKAGE_TABLE_SPACE_START;
     void *validated_end = link_target;
     lispobj symbol_name;
     char *namechars;
     boolean datap;
     void* result;
-    int strict /* If in a cold core, fail early and often. */
-      = (SymbolValue(GC_INHIBIT, 0) & WIDETAG_MASK) == UNBOUND_MARKER_WIDETAG;
-    int n = 0, m = 0;
+    int n = 0, m = 0, j;
 
-    for (head = SymbolValue(REQUIRED_RUNTIME_C_SYMBOLS,0);
-         head!=NIL; head = cdr(head), n++)
+    struct vector* symbols = VECTOR(SymbolValue(REQUIRED_RUNTIME_C_SYMBOLS,0));
+    n = fixnum_value(symbols->length);
+    for (j = 0 ; j < n ; ++j)
     {
-        lispobj item = car(head);
-        symbol_name = car(item);
-        datap = (NIL!=(cdr(item)));
-        namechars = (void*)(intptr_t)FOTHERPTR(symbol_name,vector).data;
+        lispobj item = symbols->data[j];
+        symbol_name = CONS(item)->car;
+        datap = CONS(item)->cdr != NIL;
+        namechars = (void*)(intptr_t)
+            (VECTOR(symbol_name)->data);
         result = os_dlsym_default(namechars);
         odxprint(runtime_link, "linking %s => %p", namechars, result);
 
@@ -214,21 +194,12 @@ void os_link_runtime()
                 arch_write_linkage_table_jmp(link_target,result);
         } else {
             m++;
-            if (strict)
-                fprintf(stderr,
-                        "undefined foreign symbol in cold init: %s\n",
-                        namechars);
         }
 
         link_target = (void*)(((uintptr_t)link_target)+LINKAGE_TABLE_ENTRY_SIZE);
     }
     odxprint(runtime_link, "%d total symbols linked, %d undefined",
              n, m);
-    if (strict && m)
-        /* We could proceed, but rather than run into improperly
-         * displayed internal errors, let's make ourselves heard right
-         * here and now. */
-        lose("Undefined aliens in cold init.");
 }
 #endif  /* sb-dynamic-core */
 
@@ -246,4 +217,35 @@ void os_map(int fd, int offset, os_vm_address_t addr, os_vm_size_t len)
         lose("unexpected mmap(%d, %d, ...) failure\n", addr, len);
     }
 }
+
+boolean
+gc_managed_addr_p(lispobj ad)
+{
+    struct thread *th;
+
+    if ((READ_ONLY_SPACE_START <= ad && ad < READ_ONLY_SPACE_END)
+        || (STATIC_SPACE_START <= ad && ad < STATIC_SPACE_END)
+#if defined LISP_FEATURE_IMMOBILE_SPACE
+        || (IMMOBILE_SPACE_START <= ad && ad < IMMOBILE_SPACE_END)
+#endif
+#if defined LISP_FEATURE_GENCGC
+        || (DYNAMIC_SPACE_START <= ad &&
+            ad < (DYNAMIC_SPACE_START + dynamic_space_size))
+#else
+        || (DYNAMIC_0_SPACE_START <= ad && ad < DYNAMIC_0_SPACE_END)
+        || (DYNAMIC_1_SPACE_START <= ad && ad < DYNAMIC_1_SPACE_END)
+#endif
+        )
+        return 1;
+    for_each_thread(th) {
+        if(th->control_stack_start <= (lispobj*)ad
+           && (lispobj*)ad < th->control_stack_end)
+            return 1;
+        if(th->binding_stack_start <= (lispobj*)ad
+           && (lispobj*)ad < th->binding_stack_start + BINDING_STACK_SIZE)
+            return 1;
+    }
+    return 0;
+}
+
 #endif
