@@ -201,7 +201,7 @@ static lispobj* find_sym_by_tls_index(unsigned int tls_index)
             if (widetag == SYMBOL_WIDETAG &&
                 tls_index_of(((struct symbol*)where)) == tls_index)
                 return where;
-            where += is_cons_half(header) ? 2 : sizetab[widetag](where);
+            where += OBJECT_SIZE(header, where);
         }
         if (where >= (lispobj*)DYNAMIC_SPACE_START)
             break;
@@ -405,6 +405,7 @@ static inline lispobj decode_pointer(uint32_t encoding)
 struct simple_fun* simple_fun_from_pc(char* pc)
 {
     struct code* code = (struct code*)component_ptr_from_pc((lispobj*)pc);
+    if (!code) return 0;
     struct simple_fun* prev_fun = (struct simple_fun*)
         ((char*)code + (code_header_words(code->header)<<WORD_SHIFT)
          + FIRST_SIMPLE_FUN_OFFSET(code));
@@ -456,7 +457,7 @@ static void trace1(lispobj object,
     lispobj thread_ref;
     enum ref_kind root_kind;
     struct thread* root_thread;
-    char* thread_pc;
+    char* thread_pc = 0;
     unsigned int tls_index;
     lispobj target;
     int i;
@@ -484,8 +485,14 @@ static void trace1(lispobj object,
                 fprintf(stderr, "target=%p srcs=", (void*)target);
                 while (list1) {
                     uint32_t* cell = (uint32_t*)(scratchpad->base + list1);
-                    lispobj ptr = decode_pointer(cell[0]);
-                    fprintf(stderr, "%p ", (void*)ptr);
+                    lispobj* ptr = (lispobj*)decode_pointer(cell[0]);
+                    if (hopscotch_containsp(visited, (lispobj)ptr))
+                        fprintf(stderr, "%p ", ptr);
+                    else {
+                        lispobj word = *ptr;
+                        int nwords = OBJECT_SIZE(word, ptr);
+                        fprintf(stderr, "%p+%d ", ptr, nwords);
+                    }
                     list1 = cell[1];
                 }
                 putc('\n',stderr);
@@ -575,8 +582,7 @@ static void trace1(lispobj object,
                 if (is_lisp_pointer(fun->name) &&
                     widetag_of(*native_pointer(fun->name)) == SYMBOL_WIDETAG) {
                     fprintf(file, "=");
-                    show_lstring((struct vector*)native_pointer(SYMBOL(fun->name)->name),
-                                 0, file);
+                    show_lstring(VECTOR(SYMBOL(fun->name)->name), 0, file);
                 }
             } else if (thread_pc)
                 fprintf(file, "pc=%p", thread_pc);
@@ -599,18 +605,26 @@ static void trace1(lispobj object,
         maybe_show_object_name(ptr, file);
         fprintf(file, ")%p[%d]->", (void*)ptr, next.wordindex);
         target = native_pointer(ptr)[next.wordindex];
-        if (next.wordindex == 0 &&
-            (lowtag_of(ptr) == INSTANCE_POINTER_LOWTAG  ||
-             lowtag_of(ptr) == FUN_POINTER_LOWTAG)) {
-            target = instance_layout(native_pointer(ptr));
-        }
+        // Special-case a few combinations of <type,wordindex>
+        switch (next.wordindex) {
+        case 0:
+            if (lowtag_of(ptr) == INSTANCE_POINTER_LOWTAG  ||
+                lowtag_of(ptr) == FUN_POINTER_LOWTAG)
+                target = instance_layout(native_pointer(ptr));
+            break;
+        case 1:
+            if (lowtag_of(ptr) == FUN_POINTER_LOWTAG &&
+                widetag_of(*native_pointer(ptr)) == CLOSURE_WIDETAG)
+                target -= FUN_RAW_ADDR_OFFSET;
+            break;
 #ifdef LISP_FEATURE_IMMOBILE_CODE
-          else if (lowtag_of(ptr) == OTHER_POINTER_LOWTAG &&
-                   widetag_of(FDEFN(ptr)->header) == FDEFN_WIDETAG &&
-                   next.wordindex == 3) {
-            target = fdefn_raw_referent((struct fdefn*)native_pointer(ptr));
-        }
+        case 3:
+            if (lowtag_of(ptr) == OTHER_POINTER_LOWTAG &&
+                widetag_of(FDEFN(ptr)->header) == FDEFN_WIDETAG)
+                target = fdefn_raw_referent((struct fdefn*)native_pointer(ptr));
+            break;
 #endif
+        }
         target = canonical_obj(target);
         struct layer* next_layer = top_layer->next;
         free(top_layer->nodes);
@@ -773,7 +787,7 @@ static void compute_heap_inverse(struct hopscotch_table* inverted_heap,
     uword_t scratchpad_min_size = (1 + ss.n_pointers) * 2 * sizeof (uint32_t);
     int pagesize = getpagesize();
     uword_t scratchpad_size = CEILING(scratchpad_min_size, pagesize);
-    ss.scratchpad.base = os_validate(0, scratchpad_size);
+    ss.scratchpad.base = os_allocate(scratchpad_size);
     gc_assert(ss.scratchpad.base);
     ss.scratchpad.free = ss.scratchpad.base + 2 * sizeof(uint32_t);
     ss.scratchpad.end  = ss.scratchpad.base + scratchpad_size;
@@ -953,8 +967,7 @@ struct vector* lisp_thread_name(os_thread_t os_thread)
         list = CONS(list)->cdr;
         if ((os_thread_t)lisp_thread->slots[LISP_THREAD_OS_THREAD_SLOT]
             == os_thread)
-            return (struct vector*)
-                native_pointer(lisp_thread->slots[LISP_THREAD_NAME_SLOT]);
+            return VECTOR(lisp_thread->slots[LISP_THREAD_NAME_SLOT]);
     }
     return 0;
 }
